local utils = dofile_once("mods/banana_of_rectitude/files/utils.lua")
local skill_bonuses = dofile_once("mods/banana_of_rectitude/files/skill_board/skill_bonuses.lua")

for i, entity in ipairs(EntityGetWithTag("wand")) do
  local var_store = utils.get_variable_storage_component(entity, "banana_of_rectitude_bonus_applied")
  if not var_store then
    EntityAddComponent2(entity, "VariableStorageComponent", { name = "banana_of_rectitude_bonus_applied" })
    local x, y = EntityGetTransform(entity)
    SetRandomSeed(entity, x + y)
    local EZWand = dofile_once("mods/banana_of_rectitude/lib/EZWand.lua")
    local wand = EZWand(entity)

    -- RechargeTime
    local multiplier = skill_bonuses.get_bonus("WandPercentRechargeTime") / 100
    -- print("WandPercentRechargeTime: ", tostring(multiplier))
    multiplier = utils.adjust_multiplier(wand.rechargeTime, multiplier)
    wand.rechargeTime = wand.rechargeTime * multiplier

    -- CastDelay
    local multiplier = skill_bonuses.get_bonus("WandPercentCastDelay") / 100
    -- print("WandPercentCastDelay: ", tostring(multiplier))
    multiplier = utils.adjust_multiplier(wand.castDelay, multiplier)
    wand.castDelay = wand.castDelay * multiplier

    -- ManaMax
    local multiplier = skill_bonuses.get_bonus("WandManaMax") / 100 + 1
    -- print("WandManaMax: ", tostring(multiplier))
    wand.manaMax = wand.manaMax * multiplier

    -- ManaRecharge
    local multiplier = skill_bonuses.get_bonus("WandManaRecharge") / 100
    -- print("WandManaRecharge: ", tostring(multiplier))
    multiplier = utils.adjust_multiplier(wand.manaChargeSpeed, multiplier)
    wand.manaChargeSpeed = wand.manaChargeSpeed * multiplier

    -- ChanceExtraCapacity
    local chance_extra_capacity = skill_bonuses.get_bonus("WandChanceExtraCapacity") / 100
    -- print("WandChanceExtraCapacity: ", tostring(chance_extra_capacity))
    if Randomf() < chance_extra_capacity then
      -- If someone is using some kinda mod that lets them bypass the capacity limit, let them
      if wand.capacity > 26 then
        wand.capacity = wand.capacity + 1
      else
        wand.capacity = math.min(26, wand.capacity + 1)
      end
    end

    -- Spread
    local multiplier = skill_bonuses.get_bonus("WandSpread") / 100
    -- print("WandSpread: ", tostring(multiplier))
    multiplier = utils.adjust_multiplier(wand.spread, multiplier)
    wand.spread = wand.spread * multiplier

    -- ChanceNoShuffle
    local chance_no_shuffle = skill_bonuses.get_bonus("WandChanceNoShuffle") / 100
    -- print("WandChanceNoShuffle: ", tostring(chance_no_shuffle))
    if Randomf() < chance_no_shuffle then
      wand.shuffle = false
    end
  end
end

-- Do potions too!
for i, potion_entity_id in ipairs(EntityGetWithTag("potion")) do
  local var_store = utils.get_variable_storage_component(potion_entity_id, "banana_of_rectitude_bonus_applied")
  if not var_store then
    EntityAddComponent2(potion_entity_id, "VariableStorageComponent", { name = "banana_of_rectitude_bonus_applied" })
    local material_sucker_component = EntityGetFirstComponentIncludingDisabled(potion_entity_id, "MaterialSuckerComponent")
    local barrel_size = 0
    if material_sucker_component then
      barrel_size = ComponentGetValue2(material_sucker_component, "barrel_size")
      local multiplier = skill_bonuses.get_bonus("PotionCapacityPercent") / 100 + 1
      -- print("PotionCapacityPercent: ", tostring(multiplier))
      barrel_size = barrel_size * multiplier
      ComponentSetValue2(material_sucker_component, "barrel_size", barrel_size)
    end
    local main_material = GetMaterialInventoryMainMaterial(potion_entity_id)
    AddMaterialInventoryMaterial(potion_entity_id, CellFactory_GetName(main_material), barrel_size)
  end
end
