local utils = dofile_once("mods/banana_of_rectitude/files/utils.lua")
local rectitude_meter = dofile_once("mods/banana_of_rectitude/files/rectitude_meter.lua")
local dialog_system = dofile_once("mods/banana_of_rectitude/lib/DialogSystem/dialog_system.lua")
local bonuses = dofile_once("mods/banana_of_rectitude/files/bonuses.lua")

dialog_system.functions.amogus = function(dialog)
  GamePlaySound("mods/banana_of_rectitude/audio/banana_of_rectitude.bank", "amogus", 0, 0)
end

portraits = portraits or {
  "mods/banana_of_rectitude/files/banana_portraits/portrait_1.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_2.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_3.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_4.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_5.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_6.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_7.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_8.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_9.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_10.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_11.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_12.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_13.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_14.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_15.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_16.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_17.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_18.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_19.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_20.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_21.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_22.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_23.png",
  "mods/banana_of_rectitude/files/banana_portraits/portrait_24.png",
}

function interacting(entity_who_interacted, entity_interacted, interactable_name)
  local x, y = EntityGetTransform(entity_interacted)
  SetRandomSeed(x + GameGetFrameNum(), y)
  local text = [[
    Welcome back, my friend.
    What can I do for you on this fine day?
  ]]
  local superwizard_text_extra = ""
  if GameHasFlagRun("banana_of_rectitude_superwizard_enabled") then
    superwizard_text_extra = [[
      {@pause 60}One more thing before you go: Be watchful on your journey,
      for there is an impostor {@func amogus}among us! They're armed
      and dangerous, may god have mercy on your soul shouldst
      thou run into them unawares! Keep your eyes peeled.]]
  end
  if HasFlagPersistent("banana_of_rectitude_world_blessed_with_potassium") then
    text = [[
      Thank you for freeing my monke friend.
      You are a true rectitudinarian indeed!
      What can I do for you on this fine day?
    ]]
  elseif not GameHasFlagRun("banana_of_rectitude_spoken_with_banana") then
    GameAddFlagRun("banana_of_rectitude_spoken_with_banana")
    text = [[
      Look upon my visage and rejoice, for rectitude
      hath returned to this wretched world!
    ]]
  end
  dialog_system.open_dialog({
    name = "Banana of Rectitude",
    portrait = utils.get_random_element_sequence(portraits),
    typing_sound = "banana",
    text = text,
    options = {
      {
        text = "My bananameter is full!",
        text_disabled = "(Not enough potassium)",
        enabled = function(stats)
          return rectitude_meter.get_rectitude(entity_who_interacted) >= rectitude_meter.max_rectitude
        end,
        func = function(dialog)
          dialog.show({
            text = [[
              Well done, my son! A potassic boon is in order!
              What would you like? A golden banana coin mayhaps?
              Or rather a random bananic gift?!
            ]],
            options = {
              {
                text = "Golden banan coin, please!",
                text_disabled = "(Can only gain 5 coins per run)",
                enabled = function(stats)
                  local coins_redeemed = tonumber(GlobalsGetValue("banana_of_rectitude_coins_redeemed", "0"))
                  return coins_redeemed < 5
                end,
                func = function(dialog)
                  dialog.show({
                    text = "Good choice, banana doubloons are very useful!",
                    on_closed = function()
                      rectitude_meter.set_rectitude(entity_who_interacted, 0)
                      local entity_id = GetUpdatedEntityID()
                      local x, y = EntityGetTransform(entity_id)
                      EntityLoad("mods/banana_of_rectitude/files/golden_banana_coin.xml", x + 30, y - 30)
                      local coins_redeemed = tonumber(GlobalsGetValue("banana_of_rectitude_coins_redeemed", "0"))
                      coins_redeemed = coins_redeemed + 1
                      GlobalsSetValue("banana_of_rectitude_coins_redeemed", tostring(coins_redeemed))
                    end
                  })
                end
              },
              {
                text = "I'll take the lootbox!",
                func = function(dialog)
                  dialog.show({
                    text = "Alright, enjoy, hope you get something good!",
                    on_closed = function()
                      rectitude_meter.set_rectitude(entity_who_interacted, 0)
                      utils.seed_with_frame_and_cam_pos()
                      local bonus = bonuses.pick_random_bonus()
                      GamePrintImportant(bonus.name, bonus.description or "", "mods/banana_of_rectitude/files/3piece_important_msg.png")
                      bonus.func(x, y)
                    end
                  })
                end
              },
            },
          })
        end
      },
      {
        text = "How can I become more rectitudinous?",
        func = function(dialog)
          if HasFlagPersistent("banana_of_rectitude_world_blessed_with_potassium") then
            dialog.show({
              text = [[
                Simple, just remove the source of depravity in this world.
                There are creatures calling themselves the "Hiisi", help
                them on their way to the afterlife and earn a banana!
                Eat this banana as proof of your rectitude! Mmm, justice
                has never tasted so delicious. Yum, yum!
              ]],
              options = {
                {
                  text = "I will do it!",
                  func = function(dialog)
                    dialog.show({
                      text = [[
                        Good, good! Come back when you are full of potassium!
                        A reward will be bestowed upon your return, rectitudinarian!
                      ]],
                      options = {
                        {
                          text = "Okay!"
                        }
                      }
                    })
                  end
                },
              }
            })
          else
            dialog.show({
              text = [[
                First, you must bring back a very dear friend of mine.
                He is being held captive in the domain of the execrable
                Hiisi. Deep in their base below the snowy chasms.
                Find him, free him, then returneth to me!
              ]],
              options = {
                {
                  text = "You can count on me!",
                  func = function(dialog)
                    dialog.show({
                      text = [[
                        Excellent! Godspeed, stranger, and have a safe return!
                      ]] .. superwizard_text_extra,
                      options = {
                        {
                          text = "Alright, I will, thank you!"
                        }
                      }
                    })
                  end
                },
              }
            })
          end
        end
      },
      {
        text = "I will come back later",
      },
    }
  })
end