local skill_bonuses = dofile_once("mods/banana_of_rectitude/files/skill_board/skill_bonuses.lua")

local function apply_movement_bonus(entity_id)
  local characer_platforming_comp = EntityGetFirstComponentIncludingDisabled(entity_id, "CharacterPlatformingComponent")
  if characer_platforming_comp then
    local multiplier = skill_bonuses.get_bonus("MovementSpeedPercent") / 100 + 1
    -- print("MovementSpeedPercent: ", tostring(multiplier))
    local vel_x_min = ComponentGetValue2(characer_platforming_comp, "velocity_min_x")
    local vel_x_max = ComponentGetValue2(characer_platforming_comp, "velocity_max_x")
    local run_velocity = ComponentGetValue2(characer_platforming_comp, "run_velocity")
    local fly_velocity_x = ComponentGetValue2(characer_platforming_comp, "fly_velocity_x")
    local accel_x = ComponentGetValue2(characer_platforming_comp, "accel_x")
    local accel_x_air = ComponentGetValue2(characer_platforming_comp, "accel_x_air")
    local jump_velocity_x = ComponentGetValue2(characer_platforming_comp, "jump_velocity_x")
    ComponentSetValue2(characer_platforming_comp, "velocity_min_x", vel_x_min * multiplier)
    ComponentSetValue2(characer_platforming_comp, "velocity_max_x", vel_x_max * multiplier)
    ComponentSetValue2(characer_platforming_comp, "run_velocity", run_velocity * multiplier)
    ComponentSetValue2(characer_platforming_comp, "fly_velocity_x", fly_velocity_x * multiplier)
    ComponentSetValue2(characer_platforming_comp, "accel_x", accel_x * multiplier)
    ComponentSetValue2(characer_platforming_comp, "accel_x_air", accel_x_air * multiplier)
    ComponentSetValue2(characer_platforming_comp, "jump_velocity_x", jump_velocity_x * multiplier)
  end
end

local function apply_health_bonus(entity_id)
  local damage_model_comp = EntityGetFirstComponentIncludingDisabled(entity_id, "DamageModelComponent")
  if damage_model_comp then
    local hp = ComponentGetValue2(damage_model_comp, "hp")
    local max_hp = ComponentGetValue2(damage_model_comp, "max_hp")
    local flat_bonus = skill_bonuses.get_bonus("HealthFlat")
    local multiplier = skill_bonuses.get_bonus("HealthPercent") / 100 + 1
    hp = (hp + flat_bonus / 25) * multiplier
    max_hp = (max_hp + flat_bonus / 25) * multiplier
    ComponentSetValue2(damage_model_comp, "hp", hp)
    ComponentSetValue2(damage_model_comp, "max_hp", max_hp)
  end
end

local function apply_health_percent_bonus_to_hearts()
  local content = ModTextFileGetContent("data/scripts/items/heart_better.lua")
  content = content:gsub([[max_hp %= max_hp %+ 2 %* multiplier]], [[
    local skill_bonuses = dofile_once("mods/banana_of_rectitude/files/skill_board/skill_bonuses.lua")
    local banana_multiplier = skill_bonuses.get_bonus("HealthPercent") / 100 + 1
    max_hp = max_hp + 2 * multiplier * banana_multiplier
  ]])
  ModTextFileSetContent("data/scripts/items/heart_better.lua", content)
  content = ModTextFileGetContent("data/scripts/items/heart.lua")
  content = content:gsub([[max_hp %= max_hp %+ 1 %* multiplier]], [[
    local skill_bonuses = dofile_once("mods/banana_of_rectitude/files/skill_board/skill_bonuses.lua")
    local banana_multiplier = skill_bonuses.get_bonus("HealthPercent") / 100 + 1
    max_hp = max_hp + 1 * multiplier * banana_multiplier
  ]])
  ModTextFileSetContent("data/scripts/items/heart.lua", content)
end

local function apply_gold_bonus()
  local content = ModTextFileGetContent("data/scripts/items/gold_pickup.lua")
  -- find index of "money = money + value"
  local index_of = content:find("%s*money = money %+ value")
  if index_of then
    local add_this = [[

    local rectitude_bonus_multiplier = tonumber(GlobalsGetValue("banana_of_rectitude_GoldPercent_value", "0")) / 100
    -- print("GoldPercent: ", tostring(rectitude_bonus_multiplier))
    value = value * (1 + rectitude_bonus_multiplier)]]
    content = content:sub(1, index_of - 1) .. add_this .. content:sub(index_of)
    ModTextFileSetContent("data/scripts/items/gold_pickup.lua", content)
  end
end

local function apply_perk_bonus()
  local value, count = skill_bonuses.get_bonus("PlusPerk")
  if count > 0 then
    ModLuaFileAppend("data/scripts/biomes/temple_altar.lua", "mods/banana_of_rectitude/files/temple_altar_append.lua")
    ModLuaFileAppend("data/scripts/biomes/boss_arena.lua", "mods/banana_of_rectitude/files/temple_altar_append.lua")
  end
end

local function apply_damage_bonus(entity_id)
  EntityAddComponent2(entity_id, "LuaComponent", {
    script_shot="mods/banana_of_rectitude/files/projectile_shot.lua",
    execute_every_n_frame=-1,
   })
end

local function apply_damage_reduction_bonus(entity_id)
  EntityAddComponent2(entity_id, "LuaComponent", {
    script_damage_about_to_be_received="mods/banana_of_rectitude/files/damage_about_to_be_received.lua",
    execute_every_n_frame=-1,
  })
end

local function apply_wand_and_potion_bonus(entity_id)
  EntityAddComponent2(entity_id, "LuaComponent", {
    script_source_file="mods/banana_of_rectitude/files/apply_bonus_to_entities_in_vicinity.lua",
    execute_every_n_frame=30,
  })
end


return {
  apply_movement_bonus = apply_movement_bonus,
  apply_health_bonus = apply_health_bonus,
  apply_gold_bonus = apply_gold_bonus,
  apply_perk_bonus = apply_perk_bonus,
  apply_damage_bonus = apply_damage_bonus,
  apply_damage_reduction_bonus = apply_damage_reduction_bonus,
  apply_wand_and_potion_bonus = apply_wand_and_potion_bonus,
  apply_health_percent_bonus_to_hearts = apply_health_percent_bonus_to_hearts,
}
