local utils = dofile_once("mods/banana_of_rectitude/files/utils.lua")
local bonuses = {}

local function create_banana_wand(seed_x, seed_y)
  local EZWand = dofile_once("mods/banana_of_rectitude/lib/EZWand.lua")
  local wand = EZWand("mods/banana_of_rectitude/files/banana_wand.xml")
  SetRandomSeed(seed_x, seed_y)
  wand.rechargeTime = Random(4, 20)
  wand.castDelay = Random(4, 20)
  wand.spread = Randomf(-10, 5)
  wand.manaMax = Random(400, 700)
  wand.manaChargeSpeed = Random(150, 250)
  wand.capacity = Random(12, 18)
  local spell_levels = { 1, 2, 3, 4, 5, 6, 10 }
  dofile_once("data/scripts/gun/gun_enums.lua")
  local action_types = {
    ACTION_TYPE_MODIFIER,
    ACTION_TYPE_DRAW_MANY,
    ACTION_TYPE_OTHER,
    ACTION_TYPE_UTILITY,
    ACTION_TYPE_PASSIVE,
  }
  for i=1, 2 do
    local action_id = GetRandomActionWithType(seed_x, seed_y, spell_levels[Random(1, #spell_levels)], action_types[Random(1, #action_types)], i)
    wand:AttachSpells(action_id)
  end
  for i=1, wand.capacity - 4 do
    local action_id = GetRandomAction(seed_x, seed_y, spell_levels[Random(1, #spell_levels)], i)
    wand:AddSpells(action_id)
  end
  wand:AddSpells("BANANA_OF_RECTITUDE_BANANARANG", 2, "BANANA_OF_RECTITUDE_BANANA_MISSILE", 2)
  return wand
end

table.insert(bonuses, {
  prob = 1,
  name = "Gold Rain",
  description = "Some stay dry and others feel the gain.",
  func = function(x, y)
    EntityLoad("mods/banana_of_rectitude/files/gold_spawner.xml", x, y - 100)
  end
})

table.insert(bonuses, {
  prob = 1,
  name = "Banana Phone",
  description = "Ring ring ring ring ring ring ring, you get the idea.",
  func = function(x, y)
    EntityLoad("mods/banana_of_rectitude/files/banana_phone.xml", x, y - 10)
  end,
  limit = 1
})

table.insert(bonuses, {
  prob = 1,
  name = "Random Perk",
  description = "Hope you got something good.",
  func = function(x, y)
    dofile_once("data/scripts/perks/perk.lua")
    utils.seed_with_frame_and_cam_pos()
    local random_index = Random(1, #perk_list)
    -- Make sure we don't spawn perks on top of each other
    -- Find other perks and try to find a free position
    -- What if the don't find a good position? Too bad!
    local start_x = x + 20
    for i=1, 10 do
      local did_hit, hit_x, hit_y = RaytraceSurfaces(start_x, y - 40, start_x, y + 40)
      if did_hit and #EntityGetInRadiusWithTag(hit_x, hit_y, 20, "item_perk") == 0 then
        perk_spawn(hit_x, hit_y - 10, perk_list[random_index].id, true)
        return
      else
        start_x = start_x + 20
      end
    end
  end
})

table.insert(bonuses, {
  prob = 1,
  name = "Chest",
  description = "What's in it?!",
  func = function(x, y)
    utils.seed_with_frame_and_cam_pos()
    EntityLoad("data/entities/items/pickup/chest_random.xml", x - 20, y - 10)
  end
})

table.insert(bonuses, {
  prob = 0.05,
  name = "SUPER Chest",
  description = "Oh my god! It's super! Super duper!",
  func = function(x, y)
    utils.seed_with_frame_and_cam_pos()
    EntityLoad("data/entities/items/pickup/chest_random_super.xml", x - 20, y - 10)
  end,
  limit = 1
})

table.insert(bonuses, {
  prob = 0.1,
  name = "Double Health",
  description = "Your health just doubled. Nice!",
  func = function(x, y)
    local player = EntityGetWithTag("player_unit")[1]
    if player then
      local damage_model_comp = EntityGetFirstComponentIncludingDisabled(player, "DamageModelComponent")
      if damage_model_comp then
        local hp = ComponentGetValue2(damage_model_comp, "hp")
        local max_hp = ComponentGetValue2(damage_model_comp, "max_hp")
        ComponentSetValue2(damage_model_comp, "hp", hp * 2)
        ComponentSetValue2(damage_model_comp, "max_hp", max_hp * 2)
        -- Might wanna consider max hp caps like glass cannon
        -- local max_hp_cap = ComponentGetValue2(damage_model_comp, "max_hp_cap")
        -- if max_hp_cap > 0 then
        --   max_hp_cap = math.max( max_hp, max_hp_cap )
        -- end
      end
    end
  end,
  limit = 2
})

table.insert(bonuses, {
  prob = 0.1,
  name = "Double Gold",
  description = "Capitalism, ho! Go play Recettear it's a pretty good game.",
  func = function(x, y)
    local player = EntityGetWithTag("player_unit")[1]
    if player then
      local wallet_component = EntityGetFirstComponentIncludingDisabled(player, "WalletComponent")
      if wallet_component then
        local money = ComponentGetValue2(wallet_component, "money")
        ComponentSetValue2(wallet_component, "money", money * 2)
      end
    end
  end,
  limit = 2
})

table.insert(bonuses, {
  prob = 1,
  name = "Random Spells",
  description = "Hope you got something useful.",
  func = function(x, y)
    dofile_once("data/scripts/gun/gun_actions.lua")
    utils.seed_with_frame_and_cam_pos()
    for i=1, 5 do
      local action = actions[Random(1, #actions)]
      CreateItemActionEntity(action.id, x - 30 + Random(-30, 20), y)
    end
  end
})

table.insert(bonuses, {
  prob = 0.2,
  name = "Potassium Bomb",
  description = "Things are about to get bananas.",
  func = function(x, y)
    EntityLoad("mods/banana_of_rectitude/files/banana_grenade.xml", x - 20, y)
  end
})

table.insert(bonuses, {
  prob = 5,
  name = "Bananic Ascendancy Neural Assemblage Nexus Array",
  description = "B.A.N.A.N.A.",
  func = function(x, y)
    EntityLoad("mods/banana_of_rectitude/files/skill_board/skill_board_item.xml", x - 20, y)
  end,
  limit = 1
})

table.insert(bonuses, {
  prob = 0.2,
  name = "Banana Blaster",
  description = "Pew pew, pew pew!",
  func = function(x, y)
    local wand = create_banana_wand(GameGetFrameNum(), x + y)
    wand:PlaceAt(x - 20, y)
  end,
  limit = 1
})

table.insert(bonuses, {
  prob = 1,
  name = "Bananomaly Detector",
  description = "Now you can go on an epic treasure hunt!",
  func = function(x, y)
    local wand = create_banana_wand(GameGetFrameNum(), x + y)
    wand:PlaceAt(x - 20, y)
  end,
  limit = 1
})

table.insert(bonuses, {
  prob = 0.2,
  name = "Bananaputer",
  description = "It says IBM on the chassis, which stands for Innovative Banana Machine.",
  func = function(x, y)
    EntityLoad("mods/banana_of_rectitude/files/bananaputer.xml", x - 20, y)
  end,
  limit = 1
})

for i, disk in ipairs({ "banana", "banana_2", "rectitude", "virus", "shootemup" }) do
  table.insert(bonuses, {
    prob = 1,
    name = "Save icon",
    description = "Some say it's an ancient storage device.",
    func = function(x, y)
      EntityLoad(("mods/banana_of_rectitude/files/disk_%s.xml"):format(disk), x - 20, y)
    end,
    limit = 1
  })
end

local function pick_random_bonus()
  local total_prob = 0
  for i, v in ipairs(bonuses) do
    local key = "banana_of_rectitude_bonus_picked_amount_" .. v.name
    local amount = tonumber(GlobalsGetValue(key, "0")) or 0
    if not v.limit or (amount < v.limit) then
      total_prob = total_prob + v.prob
    end
  end
  local r = Randomf(total_prob)
	for i, v in ipairs(bonuses) do
    if r <= v.prob then
      local key = "banana_of_rectitude_bonus_picked_amount_" .. v.name
      local amount = tonumber(GlobalsGetValue(key, "0")) or 0
      if not v.limit or (amount < v.limit) then
        amount = amount + 1
        GlobalsSetValue(key, tostring(amount))
        return v
      end
    end
    r = r - v.prob
	end
end

return {
  pick_random_bonus = pick_random_bonus
}
