-- dofile_once("data/scripts/lib/coroutines.lua")
local utils = dofile_once("mods/banana_of_rectitude/files/utils.lua")
local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)
local source_x, source_y = x, y
local target_x, target_y = DEBUG_GetMouseWorld()
local dist_x, dist_y = target_x - source_x, target_y - source_y
local max_segments = 50
local dist = math.sqrt(dist_x^2+dist_y^2)
local num_segments = math.min(max_segments, math.floor(dist / 5))
local segment_length_x = dist_x / num_segments
local segment_length_y = dist_y / num_segments
local turbulence = 8

if not GetValueBool("initialized", false) then
  SetValueBool("initialized", true)
  for i=1, max_segments do
    local child = EntityCreateNew()
    EntityAddChild(entity_id, child)
    EntityAddComponent2(child, "InheritTransformComponent", {})
    EntityAddComponent2(child, "SpriteComponent", {
      alpha=0.6,
      z_index=1.02,
      additive=true,
      emissive=true,
      image_file="mods/banana_of_rectitude/files/red_laser_piece.png",
      offset_x=0,
      offset_y=1.5,
      has_special_scale=true,
      special_scale_y=0.5,
    })
    local child = EntityCreateNew()
    EntityAddChild(entity_id, child)
    EntityAddComponent2(child, "InheritTransformComponent", {})
    EntityAddComponent2(child, "SpriteComponent", {
      alpha=0.6,
      z_index=1.03,
      additive=true,
      emissive=true,
      image_file="mods/banana_of_rectitude/files/yellow_laser_piece.png",
      offset_x=0,
      offset_y=1.5,
      has_special_scale=true,
      special_scale_y=0.8,
    })
    local child = EntityCreateNew()
    EntityAddChild(entity_id, child)
    EntityAddComponent2(child, "InheritTransformComponent", {})
    EntityAddComponent2(child, "SpriteComponent", {
      alpha=0.6,
      z_index=1.01,
      additive=true,
      emissive=true,
      image_file="mods/banana_of_rectitude/files/blue_laser_piece.png",
      offset_x=0,
      offset_y=1.5,
      has_special_scale=true,
      special_scale_y=0.5,
    })
  end
end

utils.seed_with_frame_and_cam_pos(entity_id)

local function vec_rotate(x, y, angle)
	local ca = math.cos(angle)
	local sa = math.sin(angle)
	local px = ca * x - sa * y
	local py = sa * x + ca * y
	return px, py
end

local orig_points = {}
for i=1, num_segments+1 do
  local p = { x = segment_length_x * (i-1), y = segment_length_y * (i-1) }
  table.insert(orig_points, { x = segment_length_x * (i-1), y = segment_length_y * (i-1) })
end
local points_red = { { x = 0, y = 0 } }
local points_yellow = { { x = 0, y = 0 } }
local points_blue = { { x = 0, y = 0 } }
for i, orig_p in ipairs(orig_points) do
  local prev_p = orig_points[i-1]
  local next_p = orig_points[i+1]
  if prev_p and next_p then
    local p_red = { x = 0, y = 0 }
    local p_yellow = { x = 0, y = 0 }
    local p_blue = { x = 0, y = 0 }
    table.insert(points_red, p_red)
    table.insert(points_yellow, p_yellow)
    table.insert(points_blue, p_blue)
    local seed_red = GameGetFrameNum() * -0.2 + i * 0.4
    local seed_yellow = GameGetFrameNum() * -0.08 + i * 0.2
    local seed_blue = GameGetFrameNum() * -0.4 + i * 0.8
    local dx = next_p.x - prev_p.x
    local dy = next_p.y - prev_p.y
    local length = math.sqrt(dx^2+dy^2)
    -- normalize
    dx = dx / length
    dy = dy / length
    -- now dx, dy is a unit vector point to the next point
    -- rotate by 90 degrees
    dx, dy = vec_rotate(dx, dy, math.rad(-90))
    -- local noise = (utils.perlin_noise(seed, seed + 0.727) - 0.5) * 2

    local noise = utils.perlin_noise(seed_red, seed_red + 0.727)
    p_red.x = orig_p.x + dx * noise * turbulence
    p_red.y = orig_p.y + dy * noise * turbulence

    local noise = utils.perlin_noise(seed_yellow, seed_yellow + 0.727)
    p_yellow.x = orig_p.x + dx * noise * turbulence * 0.5
    p_yellow.y = orig_p.y + dy * noise * turbulence * 0.5

    local noise = math.cos(seed_blue) --utils.perlin_noise(seed_blue, seed_blue + 0.727)
    p_blue.x = orig_p.x + dx * noise * turbulence * 0.7
    p_blue.y = orig_p.y + dy * noise * turbulence * 0.7
  end
end
table.insert(points_red, { x = dist_x, y = dist_y })
table.insert(points_yellow, { x = dist_x, y = dist_y })
table.insert(points_blue, { x = dist_x, y = dist_y })

local children = EntityGetAllChildren(entity_id) or {}
for i, child in ipairs(children) do
  local sprite_comp = EntityGetFirstComponentIncludingDisabled(child, "SpriteComponent")
  if sprite_comp then
    -- ComponentSetValue2(sprite_comp, "special_scale_x", 0)
    EntitySetComponentIsEnabled(child, sprite_comp, false)
  end
end

-- All this bullshit just to execute this with 1 frame delay
-- async bugs without vm_type="ONE_PER_COMPONENT_INSTANCE",
-- because with multiple entities wake_up_waiting_threads(1)
-- gets called multiple times in the same frame
state = state or {}
state[entity_id] = state[entity_id] or { children = {} }

for i, child in ipairs(state[entity_id].children) do
  EntitySetComponentIsEnabled(child.child, child.sprite_comp, true)
  ComponentSetValue2(child.sprite_comp, "special_scale_x", child.dist)
end

state[entity_id].children = {}

for i=1, num_segments do
  local ps = { points_red, points_yellow, points_blue }
  for j=1, 3 do
    local child = children[(i-1)*3+j]
    local sprite_comp = EntityGetFirstComponentIncludingDisabled(child, "SpriteComponent")
    local inherit_transform_comp = EntityGetFirstComponentIncludingDisabled(child, "InheritTransformComponent")
    if sprite_comp and inherit_transform_comp then
      local p1 = ps[j][i]
      local p2 = ps[j][i+1]
      local dx, dy = p2.x - p1.x, p2.y - p1.y
      local dist = math.sqrt(dx^2 + dy^2)
      local rot = math.atan2(dy, dx)
      ComponentSetValue2(inherit_transform_comp, "Transform", p1.x, p1.y, 1, 1, rot)
      state[entity_id].children = state[entity_id].children or {}
      table.insert(state[entity_id].children, {
        child = child,
        sprite_comp = sprite_comp,
        dist = dist,
      })
      -- async(function()
      --   ComponentSetValue2(inherit_transform_comp, "Transform", p1.x, p1.y, 1, 1, rot)
      --   wait(1)
      --   EntitySetComponentIsEnabled(child, sprite_comp, true)
      --   ComponentSetValue2(sprite_comp, "special_scale_x", dist)
      -- end)
    end
  end
end

-- if (last_frame_threads_woken or 0) < GameGetFrameNum() then
--   wake_up_waiting_threads(1)
--   last_frame_threads_woken = GameGetFrameNum()
-- end
