dofile_once("data/scripts/lib/coroutines.lua")
local utils = dofile_once("mods/banana_of_rectitude/files/utils.lua")
local entity_id = GetUpdatedEntityID()
local x, y = EntityGetTransform(entity_id)
local source_x, source_y = x, y
local target_x, target_y = DEBUG_GetMouseWorld()
local num_segments = 20

if not GetValueBool("initialized", false) then
  SetValueBool("initialized", true)
  for i=1, num_segments do
    local child = EntityCreateNew()
    EntityAddChild(entity_id, child)
    EntityAddComponent2(child, "InheritTransformComponent", {})
    EntityAddComponent2(child, "SpriteComponent", {
      alpha=0.5,
      additive=true,
      emissive=true,
      image_file="mods/banana_of_rectitude/files/blue_laser_piece.png",
      offset_x=0,
      offset_y=1.5,
      has_special_scale=true,
      update_transform=true,
      update_transform_rotation=true,
    })
    -- EntityAddComponent2(entity_id, "SpriteComponent", {
    --   alpha=0.5,
    --   additive=true,
    --   emissive=true,
    --   image_file="mods/banana_of_rectitude/files/red_laser_piece.png",
    --   offset_x=0,
    --   offset_y=2.5,
    -- })
    -- EntityAddComponent2(entity_id, "SpriteComponent", {
    --   alpha=0.5,
    --   additive=true,
    --   emissive=true,
    --   image_file="mods/banana_of_rectitude/files/blue_laser_piece.png",
    --   offset_x=0,
    --   offset_y=2,
    -- })
  end
end

utils.seed_with_frame_and_cam_pos(entity_id)
local in_seed = 0
local turbulence = 8
local speed = 0.12

local dist_x, dist_y = target_x - source_x, target_y - source_y
local segment_length_x = dist_x / num_segments
local segment_length_y = dist_y / num_segments

local function vec_rotate(x, y, angle)
	local ca = math.cos(angle)
	local sa = math.sin(angle)
	local px = ca * x - sa * y
	local py = sa * x + ca * y
	return px, py
end

local orig_points = {}
for i=1, num_segments+1 do
  local p = { x = segment_length_x * (i-1), y = segment_length_y * (i-1) }
  table.insert(orig_points, { x = segment_length_x * (i-1), y = segment_length_y * (i-1) })
end
local points = { { x = 0, y = 0 } }
for i, orig_p in ipairs(orig_points) do
  local prev_p = orig_points[i-1]
  local next_p = orig_points[i+1]
  if prev_p and next_p then
    local p = { x = 0, y = 0 }
    table.insert(points, p)
    local seed = GameGetFrameNum() * 0.2 + i * 5
    local dx = next_p.x - prev_p.x
    local dy = next_p.y - prev_p.y
    local length = math.sqrt(dx^2+dy^2)
    -- normalize
    dx = dx / length
    dy = dy / length
    -- now dx, dy is a unit vector point to the next point
    -- rotate by 90 degrees
    dx, dy = vec_rotate(dx, dy, math.rad(-90))

    local noise = utils.perlin_noise(seed, seed + 0.727)
    -- local noise = (utils.perlin_noise(seed, seed + 0.727) - 0.5) * 2
    p.x = orig_p.x + dx * noise * turbulence
    p.y = orig_p.y + dy * noise * turbulence
  end
end
table.insert(points, { x = dist_x, y = dist_y })

local children = EntityGetAllChildren(entity_id) or {}
for i=1, num_segments do
  local child = children[i]
  local sprite_comp = EntityGetFirstComponentIncludingDisabled(child, "SpriteComponent")
  local inherit_transform_comp = EntityGetFirstComponentIncludingDisabled(child, "InheritTransformComponent")
  if sprite_comp and inherit_transform_comp then
    local p1 = points[i]
    local p2 = points[i+1]
    local dx, dy = p2.x - p1.x, p2.y - p1.y
    local dist = math.sqrt(dx^2 + dy^2)
    local rot = math.atan2(dy, dx)
    async(function()
      ComponentSetValue2(inherit_transform_comp, "Transform", p1.x, p1.y, 1, 1, rot)
      wait(1)
      ComponentSetValue2(sprite_comp, "special_scale_x", dist)
    end)
  end
end
