local utils = dofile_once("mods/banana_of_rectitude/files/utils.lua")
local rectitude_meter = dofile_once("mods/banana_of_rectitude/files/rectitude_meter.lua")
local dialog_system = dofile_once("mods/banana_of_rectitude/lib/DialogSystem/dialog_system.lua")

local function bless_world(entity_who_interacted)
  GamePrintImportant("The world has been blessed with potassium!", "Terraria reference", "mods/banana_of_rectitude/files/3piece_important_msg.png")
  local player = EntityGetWithTag("player_unit")[1]
  if player then
    local x, y = EntityGetTransform(player)
    SetRandomSeed(x, y)
    for i=1, 8 do
      EntityLoad("mods/banana_of_rectitude/files/banana_physics_prop.xml", x + Random(-50, 50), y + Random(-30 , -80))
    end
  end
  local found = false
  for i, comp in ipairs(EntityGetComponentIncludingDisabled(entity_who_interacted, "LuaComponent") or {}) do
    if ComponentGetValue2(comp, "script_source_file") == "mods/banana_of_rectitude/files/consume_potassium.lua" then
      found = true
      break
    end
  end
  if not found then
    EntityAddComponent2(entity_who_interacted, "LuaComponent", {
      script_source_file="mods/banana_of_rectitude/files/consume_potassium.lua",
      execute_every_n_frame=5,
    })
    rectitude_meter.set_rectitude(entity_who_interacted, 0)
  end
  AddFlagPersistent("banana_of_rectitude_world_blessed_with_potassium")
  GlobalsSetValue("banana_of_rectitude_le_monke_dialog", "1")
  GamePlaySound("data/audio/Desktop/event_cues.bank", "event_cues/new_biome/create", 0, 0)
end

local dialogs_after_finished = {
  {
    text = [[
      What are you waiting for? Go out into the world
      and collect that sweet sweet potassium mmmm yea.
    ]],
  },
  {
    text = [[
      Still talking to me? Am I this charming?
    ]],
  },
  {
    text = [[
      Look, I have nothing else to say. I may be
      a smart monkey, but still just a monkey.
    ]],
  },
  {
    text = [[
      You're saying I'm not a monkey but a great ape?
      ... My god, you just gave me an identity crisis.
    ]],
  },
  {
    text = [[
      Leave me alone, I need some time to think...
    ]],
  },
}

local biome_specific_dialog = {}
biome_specific_dialog["$biome_coalmine"] = {
  text = [[
    ... Oops, the ground seems to have opened up beneath me!
    Oh well, at least there's some cute little spider friends
    to keep me company.
  ]],
}
biome_specific_dialog["$biome_excavationsite"] = {
  text = [[
    How did we end up here? Are you doing this on purpose?
  ]],
}
biome_specific_dialog["$biome_holymountain"] = {
  text = [[
    Uh oh, the holy place has been defiled.
    Steve's gonna be pissed.
  ]],
}
biome_specific_dialog["$biome_snowcave"] = {
  text = [[
    Brrr so cold, it reminds me of my old prison!
    Wait a second... where are we going?!
  ]],
}
biome_specific_dialog["$biome_snowcastle"] = {
  text = [[
    #Nooooo!!!# I'm back here?! This place brings back
    bad memories... please get me out of here!
  ]],
}
biome_specific_dialog["$biome_rainforest"] = {
  text = [[
    HOME! Beautiful home fauna oh how I missed you!
    Too bad the lukki have killed all my monkey friends.
    The bananas are gone too... Thanks to the hiisi, no doubt!
  ]],
}
biome_specific_dialog["$biome_vault"] = {
  text = [[
    Welcome to the tetanus zone.
  ]],
}
biome_specific_dialog["$biome_crypt"] = {
  text = [[
    Temple of the art?
    More like temple of the fart, am I right?!
    \*farts\*
  ]],
}
biome_specific_dialog["$biome_lava"] = {
  text = [[
    How the hell are we ever going to make it back?
  ]],
}
biome_specific_dialog["$biome_boss_victoryroom"] = {
  text = [[
    I've been a good boy, why am I in hell?!
    I guess the old saying is true:
    "You go to hell before you die!"
  ]],
}

function interacting(entity_who_interacted, entity_interacted, interactable_name)
  local dialog_index = tonumber(GlobalsGetValue("banana_of_rectitude_le_monke_dialog", "0"))
  if HasFlagPersistent("banana_of_rectitude_world_blessed_with_potassium") then
    local x, y = EntityGetTransform(entity_interacted)
    local biome_name = BiomeMapGetName(x, y)
    if biome_specific_dialog[biome_name] then
      if biome_name == "$biome_boss_victoryroom" then
        AddFlagPersistent("banana_of_rectitude_lemonke_been_to_hell")
      end
      dialog_system.open_dialog(utils.merge_tables_pairs({
        name = "The patron saint of potassium",
        portrait = "mods/banana_of_rectitude/files/lemonke_portrait.png",
        typing_sound = "monke"
      }, biome_specific_dialog[biome_name]))
    else
      local stone_option
      if GameHasFlagRun("banana_of_rectitude_portal_stone_gotten") then
        stone_option = {
          text = "Enjoy your freedom, compadre!",
          func = function(dialog)
            dialog.show({
              text = [[
                I will, thanks!
              ]],
            })
          end
        }
      else
        stone_option = {
          text = "Do you have anything that can help me?",
          func = function(dialog)
            dialog.show({
              text = [[
                Hmm... yes, actually! This stone can help you create a
                shortcut between our magnificent benefactor and yourself!
              ]],
              options = {
                {
                  text = "Thanks!"
                }
              },
              on_closed = function()
                local x, y = EntityGetTransform(entity_interacted)
                EntityLoad("mods/banana_of_rectitude/files/banana_hearthstone.xml", x + 15, y - 15)
                GameAddFlagRun("banana_of_rectitude_portal_stone_gotten")
              end
            })
          end
        }
      end
      dialog_system.open_dialog({
        name = "The patron saint of potassium",
        portrait = "mods/banana_of_rectitude/files/lemonke_portrait.png",
        typing_sound = "monke",
        text = [[
          Thank you again for freeing me, the air up here
          is so much nicer than in those dank caves.{@pause 30}
          Mmm yeah{@delay 30}...{@delay 3} \*sniff\*{@pause 45} smells like ~FREEDOM~, brother!
        ]],
        options = {
          stone_option
        },
      })
    end
  elseif dialog_index == 0 then
    dialog_system.open_dialog({
      name = "The patron saint of potassium",
      portrait = "mods/banana_of_rectitude/files/lemonke_portrait.png",
      typing_sound = "monke",
      text = [[
        #UGH UHGH UUUGH AAAAAAAA AAAHHHHHHHHHHH!!!!{@pause 60}
        \*cough\* {@pause 30}\*cough\*#{@pause 30}
        Excuse me, I had something stuck in my throat.
      ]],
      options = {
        {
          text = "...",
          func = function(dialog)
            dialog.show({
              text = [[
                #FREEEDOOOM!!!#{@pause 30} At last! {@pause 30}For thousands of years I have
                been held captive in this cage, {@pause 30}courtesy of the damnable hiisi!{@pause 30}
                Allow me to introduce myself, {@pause 30}I am the patron saint of
                potassium and the first rectitudinarian.
              ]],
              options = {
                {
                  text = "...Okay",
                  func = function(dialog)
                    dialog.show({
                      text = [[
                        Long has the power of potassium been sought after by the
                        hiisi, even after I told them its secret, they kept
                        torturing me, for they did not believe me when I said:
                        ~"Potassium only reveals itself to the rectitudinous!"~
                      ]],
                      options = {
                        {
                          text = "...Uh huh",
                          func = function(dialog)
                            dialog.show({
                              text = [[
                                By freeing me, you have proven your potential.
                                Do you wish to join the potassic covenant and become
                                a rectitudinarian like myself?
                              ]],
                              options = {
                                {
                                  text = "Hell YES!!!!!",
                                  func = function(dialog)
                                    dialog.show({
                                      text = [[
                                        ~BY THE POWER OF POTASSIUM{@delay 20}!!!~{@delay 3}
                                        Let there be banan! {@pause 30}.{@pause 30}.{@pause 30}.{@pause 30} It is done!
                                        Venture forth and gather rectitude (potassium)!
                                      ]],
                                      options = {
                                        {
                                          text = "Cool, thanks, bye!"
                                        }
                                      },
                                      on_closed = function() bless_world(entity_who_interacted) end,
                                    })
                                  end
                                },
                              }
                            })
                          end
                        },
                      }
                    })
                  end
                },
              }
            })
          end
        }
      }
    })
  else
    dialog_system.open_dialog(utils.merge_tables_pairs({
      name = "The patron saint of potassium",
      portrait = "mods/banana_of_rectitude/files/lemonke_portrait.png",
      typing_sound = "monke",
      on_closed = function()
        if dialog_index < #dialogs_after_finished then
          GlobalsSetValue("banana_of_rectitude_le_monke_dialog", tostring(dialog_index + 1))
        end
      end
    }, dialogs_after_finished[dialog_index]))
  end
end
