local max_rectitude = 100

local function get_variable_storage_component(entity_id, name)
  for i, comp in ipairs(EntityGetComponentIncludingDisabled(entity_id, "VariableStorageComponent") or {}) do
    if ComponentGetValue2(comp, "name") == name then
      return comp
    end
  end
end

local function get_banana_ingest_amount(entity_id)
  local material_inventory_comp = EntityGetFirstComponentIncludingDisabled(entity_id, "MaterialInventoryComponent")
  if material_inventory_comp then
    local count_per_material_type = ComponentGetValue2(material_inventory_comp, "count_per_material_type")
    return count_per_material_type[CellFactory_GetType("meat_banana") + 1]
  end
end

-- Adds the specified amount to the var_store on the entity and returns the new amount
local function add_rectitude(entity_id, amount)
  local var_store = get_variable_storage_component(entity_id, "banana_of_rectitude_rectitude")
  if not var_store then
    var_store = EntityAddComponent2(entity_id, "VariableStorageComponent", {
      name = "banana_of_rectitude_rectitude",
      value_float = 0
    })
  end
  local current_amount = ComponentGetValue2(var_store, "value_float")
  current_amount = math.min(max_rectitude, current_amount + amount)
  ComponentSetValue2(var_store, "value_float", current_amount)
  return current_amount
end

local function get_rectitude(entity_id)
  local var_store = get_variable_storage_component(entity_id, "banana_of_rectitude_rectitude")
  if not var_store then
    return 0
  end
  return ComponentGetValue2(var_store, "value_float")
end

local function set_rectitude(entity_id, amount)
  local var_store = get_variable_storage_component(entity_id, "banana_of_rectitude_rectitude")
  if not var_store then
    var_store = EntityAddComponent2(entity_id, "VariableStorageComponent", {
      name = "banana_of_rectitude_rectitude",
      value_int = 0
    })
  end
  ComponentSetValue2(var_store, "value_float", amount)
end

local ingestion_satiation_per_cell = MagicNumbersGetValue("INGESTION_SATIATION_PER_CELL")
-- returns the amount converted
local function convert_banana_meat_to_rectitude(entity_id, amount)
  local banana_in_stomach = get_banana_ingest_amount(entity_id)
  if banana_in_stomach > 0 then
    -- Remove all of it by setting the stomach contents to 0
    local take_amount = math.min(banana_in_stomach, amount)
    banana_in_stomach = banana_in_stomach - take_amount
    AddMaterialInventoryMaterial(entity_id, "meat_banana", banana_in_stomach)
    local multiplier = tonumber(GlobalsGetValue("banana_of_rectitude_PotassiumPercent_value", "0")) / 100
    add_rectitude(entity_id, take_amount * 0.1 * (1 + multiplier))
    -- Make banana not take up stomach size
    local ingestion_component = EntityGetFirstComponentIncludingDisabled(entity_id, "IngestionComponent")
    if ingestion_component then
      local ingestion_size = ComponentGetValue2(ingestion_component, "ingestion_size")
      ingestion_size = ingestion_size - take_amount * ingestion_satiation_per_cell
      ComponentSetValue2(ingestion_component, "ingestion_size", ingestion_size)
    end
    return take_amount
  end
  return 0
end

local function has_rectitude_unlocked(entity_id)
  return get_variable_storage_component(entity_id, "banana_of_rectitude_rectitude") ~= nil
end

return {
  max_rectitude = max_rectitude,
  convert_banana_meat_to_rectitude = convert_banana_meat_to_rectitude,
  get_rectitude = get_rectitude,
  set_rectitude = set_rectitude,
  add_rectitude = add_rectitude,
  has_rectitude_unlocked = has_rectitude_unlocked,
}
