-- These are just estimates
local screen_width = 640
local screen_height = 340

local function get_direction( x1, y1, x2, y2 )
  return math.atan2( ( y2 - y1 ), ( x2 - x1 ) )
end

local function get_distance( x1, y1, x2, y2 )
  local result = math.sqrt( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )
  return result
end

local function new_shot(x, y, vx, vy)
  return {
    x = x, y = y, vx = vx or 3, vy = vy or 0,
    tick = 0, chars = { "o", "O" }, current_char = 0,
    update = function(self)
      self.x = self.x + self.vx
      self.y = self.y + self.vy
      self.tick = self.tick + 1
      if self.tick > 15 then
        self.tick = 0
        self.current_char = (self.current_char + 1) % #self.chars
      end
    end,
    render = function(self)
      GuiText(gui, self.x, self.y, self.chars[self.current_char+1])
    end
  }
end

local function new_enemy(x, y)
  return {
    x = x, y = y, vx = -1.5, vy = 0,
    tick = 0, shot_delay = 30, shot_speed = 3,
    update = function(self)
      self.x = self.x + self.vx
      self.y = self.y + self.vy
      self.tick = self.tick + 1
      if self.tick > self.shot_delay then
        self:shoot()
        self.tick = 0
        self.shot_delay = 45 + math.random() * 30
      end
      if self.x < -20 then
        self.x = screen_width
        self.y = math.random() * screen_height
      end
    end,
    render = function(self)
      GuiText(gui, self.x + 5, self.y - 5, "/")
      GuiText(gui, self.x, self.y, "<=")
      GuiText(gui, self.x + 5, self.y + 5, "\\")
    end,
    shoot = function(self)
      local dir = get_direction(self.x, self.y, ship.x, ship.y)
      table.insert(enemy_shots, {
        x = self.x, y = self.y, vx = math.cos(dir) * self.shot_speed, vy = math.sin(dir) * self.shot_speed, damage = 1,
        update = function(self)
          self.x = self.x + self.vx
          self.y = self.y + self.vy
        end,
        render = function(self)
          GuiText(gui, self.x, self.y, "*")
        end
      })
    end
  }
end

local function new_explosion(x, y)
  return {
    x = x, y = y, current_frame = 1,
    tick = 0,
    update = function(self)
      self.tick = self.tick + 1
      if self.tick % 7 == 0 then
        self.current_frame = self.current_frame + 1
        if self.current_frame > 5 then
          return true
        end
      end
    end,
    render = function(self)
      if self.current_frame == 1 then
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x, self.y, "*")
      elseif self.current_frame == 2 then
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x - 5, self.y, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x + 5, self.y, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x, self.y - 5, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x, self.y + 5, "*")
      elseif self.current_frame == 3 then
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x - 10, self.y, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x + 10, self.y, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x, self.y - 10, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x, self.y + 10, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x - 7, self.y - 7, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x + 7, self.y + 7, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x - 7, self.y + 7, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x + 7, self.y - 7, "*")
      elseif self.current_frame == 4 then
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x - 12, self.y, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x + 12, self.y, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x, self.y - 12, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x, self.y + 12, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x - 9, self.y - 9, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x + 9, self.y + 9, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x - 9, self.y + 9, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x + 9, self.y - 9, "*")
      elseif self.current_frame == 5 then
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x - 13, self.y, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x + 13, self.y, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x, self.y - 13, "*")
        GuiColorSetForNextWidget(gui, 1, 1, 0, 1)
        GuiText(gui, self.x, self.y + 13, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x - 10, self.y - 10, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x + 10, self.y + 10, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x - 10, self.y + 10, "*")
        GuiColorSetForNextWidget(gui, 1, 0.4, 0, 1)
        GuiText(gui, self.x + 10, self.y - 10, "*")
      end
    end,
  }
end

local function new_giant_explosion(x, y)
  return {
    x = x, y = y, current_frame = 1,
    tick = 0,
    update = function(self)
      self.tick = self.tick + 1
      if self.tick > 120 then
        return true
      end
    end,
    render = function(self)
      for r=math.max(0, math.floor(self.tick / 3) - 20),math.min(12, math.floor(self.tick / 3)) do
        for i=1, r*10 do
          GuiColorSetForNextWidget(gui, 1, 1 - (i / 120), 0, 1)
          GuiText(gui, self.x + math.cos(i + math.random()*0.1)*r*20, self.y + math.sin(i + math.random()*0.1)*r*20, "*")
        end
      end
    end,
  }
end

local function new_powerup(x, y, powerup_type)
  local types = {
    speed_upgrade = "( >>)",
    shot_upgrade = "( + )",
    bomb = "<[B]>",
  }
  return {
    x = x, y = y, vx = -1, vy = 0, type = powerup_type,
    update = function(self)
      self.x = self.x + self.vx
      self.y = self.y + self.vy
    end,
    render = function(self)
      GuiText(gui, self.x, self.y, types[self.type])
    end
  }
end

local function new_bomb(x, y)
  return {
    x = x, y = y, vx = 2, vy = 0,
    tick = 0,
    update = function(self)
      self.x = self.x + self.vx
      self.y = self.y + self.vy
      self.tick = self.tick + 1
      if self.tick > 120 then
        return true
      end
    end,
    render = function(self)
      GuiText(gui, self.x-2, self.y-2, "#")
      GuiText(gui, self.x+2, self.y-2, "#")
      GuiText(gui, self.x-2, self.y+2, "#")
      GuiText(gui, self.x+2, self.y+2, "#")
    end
  }
end

local function OnPausedChanged( is_paused, is_inventory_pause )
  if is_paused then
    game_over = false
    shots = {}
    enemy_shots = {}
    enemies = {}
    explosions = {}
    powerups = {}
    falling_bombs = {}
    giant_explosions = {}
    score = 0
    ship = {
      x = 0, y = 0, speed = 2, hp = 3, hp_max = 3, num_shots = 1, bombs = 3, bombs_max = 3,
      update = function(self, sx, sy)
        local dir = get_direction(self.x, self.y, sx, sy)
        local dist = get_distance(self.x, self.y, sx, sy)
        if dist < self.speed then
          self.x = sx
          self.y = sy
        else
          self.x = self.x + math.cos(dir) * self.speed
          self.y = self.y + math.sin(dir) * self.speed
        end
      end,
      render = function(self)
        GuiText(gui, self.x-5, self.y-5, "-")
        GuiText(gui, self.x, self.y-10, "\\")
        local clicked, right_clicked = GuiButton(gui, 99999, self.x, self.y, "D>>")
        if clicked then
          self:shoot()
        end
        if right_clicked then
          self:drop_bomb()
        end
        GuiText(gui, self.x, self.y+10, "/")
        GuiText(gui, self.x-5, self.y+5, "-")
      end,
      shoot = function(self)
        local num_straight_shots = math.min(3, self.num_shots)
        local num_angled_shots = math.min(2, self.num_shots - num_straight_shots)
        for i=1, num_straight_shots do
          table.insert(shots, new_shot(self.x, self.y - (num_straight_shots-1) * 2.5 + (i-1) * 5))
        end
        for i=1, num_angled_shots do
          table.insert(shots, new_shot(self.x, self.y + 30 + -i * 20, 3, -1 + (i % 2) * 2))
        end
      end,
      drop_bomb = function(self)
        if self.bombs > 0 then
          self.bombs = self.bombs - 1
          table.insert(falling_bombs, new_bomb(self.x, self.y))
        end
      end,
      take_damage = function(self, damage)
        self.hp = self.hp - damage
        if self.hp <= 0 then
          table.insert(explosions, new_explosion(self.x, self.y))
          game_over = true
        end
      end,
      pickup = function(self, item)
        if item.type == "shot_upgrade" then
          self.num_shots = self.num_shots + 1
        elseif item.type == "speed_upgrade" then
          self.speed = math.min(5.0, self.speed + 0.5)
        elseif item.type == "bomb" then
          self.bombs = math.min(self.bombs_max, self.bombs + 1)
        end
      end
    }
  end
end

local utils = dofile_once("mods/banana_of_rectitude/files/utils.lua")
local function OnPausePreUpdate(gui)
  screen_width, screen_height = GuiGetScreenDimensions(gui)
  local sx, sy = utils.get_mouse_gui_pos(gui)
  if not game_over then
    ship:update(sx - 5, sy - 7)
    ship:render()

    GuiText(gui, 5, 300, "Health: [" .. ("="):rep(ship.hp) .. ("  "):rep(ship.hp_max - ship.hp) .. "]")
    GuiText(gui, 5, 310, "Bombs: " .. ("*"):rep(ship.bombs))
    GuiText(gui, 5, 320, "Score: " .. score)

    for i, powerup in ipairs(powerups) do
      powerup:update()
      powerup:render()
      if powerup.x < -20 then
        table.remove(powerups, i)
      end
      if get_distance(powerup.x + 10, powerup.y, ship.x, ship.y) < 7 then
        ship:pickup(powerup)
        table.remove(powerups, i)
      end
    end

    for i, shot in ipairs(shots) do
      shot:update()
      if shot.x < -20 or shot.x > screen_width + 20 or shot.y < -20 or shot.y > screen_height + 20 then
        table.remove(shots, i)
      else
        shot:render()
      end
    end

    for i, enemy in ipairs(enemies) do
      if enemy:update() then
        table.remove(enemies, i)
      else
        local died = false
        for i2, shot in ipairs(shots) do
          if get_distance(shot.x, shot.y, enemy.x, enemy.y) < 5 then
            table.insert(explosions, new_explosion(enemy.x, enemy.y))
            if math.random() < 0.1 then
              local roll = math.random()
              if roll < 0.4 then
                table.insert(powerups, new_powerup(enemy.x, enemy.y, "shot_upgrade"))
              elseif roll < 0.8 then
                table.insert(powerups, new_powerup(enemy.x, enemy.y, "speed_upgrade"))
              else
                table.insert(powerups, new_powerup(enemy.x, enemy.y, "bomb"))
              end
            end
            table.remove(enemies, i)
            table.remove(shots, i2)
            died = true
            score = score + 50
          end
        end
        if not died then
          enemy:render()
        end
      end
    end

    for i, shot in ipairs(enemy_shots) do
      shot:update()
      if shot.x < -20 or shot.x > screen_width + 20 or shot.y < -20 or shot.y > screen_height + 20 then
        table.remove(enemy_shots, i)
      elseif get_distance(shot.x, shot.y, ship.x, ship.y) < 6 then
        ship:take_damage(shot.damage)
        table.remove(enemy_shots, i)
      else
        shot:render()
      end
    end
  end
  for i, explosion in ipairs(explosions) do
    if explosion:update() then
      table.remove(explosions, i)
    else
      explosion:render()
    end
  end

  -- print("#giant_explosions: " .. tostring(#giant_explosions))
  for i, giant_explosion in ipairs(giant_explosions) do
    if giant_explosion:update() then
      table.remove(giant_explosions, i)
    else
      giant_explosion:render()
    end
  end

  -- print("#falling_bombs: " .. tostring(#falling_bombs))
  for i, bomb in ipairs(falling_bombs) do
    if bomb:update() then
      table.remove(falling_bombs, i)
      table.insert(giant_explosions, new_giant_explosion(bomb.x, bomb.y))
      enemies = {}
      enemy_shots = {}
    else
      bomb:render()
    end
  end

  if not game_over then
    tick = (tick or 0) + 1
    -- Spawn enemies
    if tick % math.floor(math.max(20, 120 - (score / 100 / 50) * 60)) == 0 then
      table.insert(enemies, new_enemy(screen_width, math.random() * screen_height))
    end
  else
    game_over_phase = (game_over_phase or 0) + 0.02
    for i=1, 18 do
      GuiText(gui, 50 + math.cos(game_over_phase + i * 20) * 30, i * 20 - 15, "G A M E   O V E R")
    end
    GuiText(gui, 180, 80, "Your score: " .. tostring(score))
  end
end

return {
  OnPausePreUpdate = OnPausePreUpdate,
  OnPausedChanged = OnPausedChanged,
}