local utils = dofile_once("mods/banana_of_rectitude/files/utils.lua")
dofile_once("data/scripts/lib/utilities.lua")
local EZMouse = dofile_once("mods/banana_of_rectitude/lib/EZMouse/EZMouse.lua")("mods/banana_of_rectitude/lib/EZMouse/")
local scale = 1
local board_width, board_height = 200, 200
local board_x, board_y = 100, 100
local start_x, start_y = 0, 0
local center_x, center_y = 498, 513
local scroll_offset_x, scroll_offset_y-- = -(center_x * scale - board_width / 2), -(center_y * scale - board_height / 2)
dofile_once("mods/banana_of_rectitude/files/skill_board/json.lua")
local data = JSON:decode(dofile_once("mods/banana_of_rectitude/files/skill_board/nodes.lua"))

local b = dofile_once("mods/banana_of_rectitude/files/skill_board/skill_bonuses.lua")
local bonuses = b.bonuses
local display_order = b.display_order

-- Adds a + in front of a number string if its positive,
-- so the result always has a sign in front of it
local function sign_str(num)
  return num > 0 and ("+" .. tostring(num)) or tostring(num)
end

local function activate_node(node, save)
  if save == nil then
    save = true
  end

  -- Don't apply bonuses twice in case it gets called again
  if not node.activated then
    -- local value, count = b.get_bonus(node.bonus.type)
    -- b.set_bonus(node.bonus.type, value + (node.bonus.value or 0), count + 1)
    node.activated = true
    bonuses[node.bonus.type].count = bonuses[node.bonus.type].count + 1
    bonuses[node.bonus.type].value = bonuses[node.bonus.type].value + (node.bonus.value or 0)
  end

  if save then
    -- Save which nodes have been activated persistently
    local out = ""
    for i, v in ipairs(data.nodes) do
      out = out .. (v.activated and "1" or "0")
      if next(data.nodes, i) then
        out = out .. ";"
      end
    end
    ModSettingSet("banana_of_rectitude.activated_skill_nodes", out)
  end
end

-- Activate all nodes that are defined as already active in the nodes.lua
for i, v in ipairs(data.nodes) do
  v.bonus = data.bonuses[v.bonus + 1]
  if v.pre_activated then
    activate_node(v, false)
  end
end

for i, active in ipairs(b.get_activated_node_indexes()) do
  if active then
    activate_node(data.nodes[i], false)
  end
end

local function get_bonus_tootlip(bonus)
  local b = bonuses[bonus.type]
  if b.display_func then
    return b.display_func(bonus, b.display_text, sign_str(bonus.value or 0))
  else
    return string.format(b.display_text, sign_str(bonus.value or 0))
  end
end

local function constrain_offset()
  scroll_offset_x = utils.clamp(scroll_offset_x, -1000 * scale + board_width, 0)
  scroll_offset_y = utils.clamp(scroll_offset_y, -1000 * scale + board_height, 0)
end

local widget = EZMouse.Widget({
  x = 100, y = 100, z = 1,
  width = 200, height = 200,
  draggable = true, enabled = true,
})
widget:AddEventListener("drag_start", function(self, event)
  start_x = self.x
  start_y = self.y
end)
widget:AddEventListener("drag_end", function(self, event)
  self.x = start_x
  self.y = start_y
end)
widget:AddEventListener("drag", function(self, event)
  scroll_offset_x = scroll_offset_x + event.dx
  scroll_offset_y = scroll_offset_y + event.dy
  constrain_offset()
end)

local function neighbor_is_active(node)
  for i, connection in ipairs(node.connections) do
    if data.nodes[connection.from].activated or data.nodes[connection.to].activated then
      return true
    end
  end
  return false
end

local function do_rects_overlap(x1, y1, w1, h1, x2, y2, w2, h2)
  return not (x1 + w1 <= x2 or x2 + w2 <= x1 or y1 + h1 <= y2 or y2 + h2 <= y1)
end

return function(gui, gui2, x, y, width, height)
  board_x = x
  board_y = y
  board_width = width
  board_height = height
  widget.x = x
  widget.y = y
  widget.width = width
  widget.height = height
  if not scroll_offset_x then
    scroll_offset_x, scroll_offset_y = -(center_x * scale - board_width / 2), -(center_y * scale - board_height / 2)
  end
  -- local mx, my = utils.get_mouse_gui_pos(gui)
  -- Save a reference otherwise the widget gets garbage collected (genius design by myself)
  local widget = widget
  -- GuiStartFrame(gui)
  GuiOptionsAdd(gui, GUI_OPTION.NoPositionTween)
  EZMouse.update(gui)
  -- widget:DebugDraw(gui)

  local id = 1
  local function new_id()
    id = id + 1
    return id
  end

  local perk_points = tonumber(GlobalsGetValue("banana_of_rectitude_perk_points", "0")) or 0

  GuiZSetForNextWidget(gui, 11)
  GuiBeginScrollContainer(gui, new_id() + GameGetFrameNum() % 2 + 34745, board_x, board_y, board_width, board_height, false, 0, 0)
  GuiZSetForNextWidget(gui, 10)
  GuiOptionsAddForNextWidget(gui, GUI_OPTION.NonInteractive)
  GuiImage(gui, new_id(), scroll_offset_x, scroll_offset_y, "mods/banana_of_rectitude/files/skill_board/skill_board2.png", 1, scale, scale)

  local proto_click_catchers = {}

  for i, node in ipairs(data.nodes) do
    local node_draw_pos_x = node.x * scale + scroll_offset_x
    local node_draw_pos_y = node.y * scale + scroll_offset_y
    local button_width, button_height = 20, 20
    if do_rects_overlap(node_draw_pos_x, node_draw_pos_y, button_width, button_height, 0, 0, board_width, board_height) then
      local button_img = "mods/banana_of_rectitude/files/skill_board/button_big.png"
      if not neighbor_is_active(node) then
        button_img = "mods/banana_of_rectitude/files/skill_board/button_big_inactive.png"
      elseif node.activated then
        GuiColorSetForNextWidget(gui, 0.5, 1, 0.3, 1)
      end
      GuiZSetForNextWidget(gui, 9)
      GuiImage(gui, new_id(), node_draw_pos_x, node_draw_pos_y, button_img, 1, scale, scale)
      local clicked, right_clicked, hovered, x, y, width, height, draw_x, draw_y, draw_width, draw_height = GuiGetPreviousWidgetInfo(gui)
      table.insert(proto_click_catchers, { x = x, y = y, node = node })
      GuiZSetForNextWidget(gui, 8.9)
      local icon_file = ("mods/banana_of_rectitude/files/skill_board/icons/%s%s.png"):format(not node.activated and "inactive/" or "", node.bonus.type)
      GuiImage(gui, new_id(), node_draw_pos_x, node_draw_pos_y, icon_file, 1, scale, scale)
    end
  end
  local function scale_pos(x, y, old_scale, new_scale, pivot_x, pivot_y)
    pivot_x = pivot_x - board_x
    pivot_y = pivot_y - board_y
    local scale_change = new_scale / old_scale
    local distance_from_pivot_x = x - pivot_x
    local distance_from_pivot_y = y - pivot_y
    x = pivot_x + distance_from_pivot_x * scale_change
    y = pivot_y + distance_from_pivot_y * scale_change
    return x, y
  end
  GuiEndScrollContainer(gui)

  -- Banana coin display
  local amount_text = ("x %d"):format(perk_points)
  local text_width, text_height = GuiGetTextDimensions(gui, amount_text)
  GuiLayoutBeginHorizontal(gui, board_x + board_width - 16 - text_width - 4, board_y - 22, true)
  GuiImage(gui, new_id(), 0, 0, "mods/banana_of_rectitude/files/banana_coin.png", 1, 1, 1)
  -- local clicked, right_clicked, hovered, x, y, width, height, draw_x, draw_y, draw_width, draw_height = GuiGetPreviousWidgetInfo(gui)
  GuiText(gui, 0, 8 - text_height / 2, amount_text)
  local clicked, right_clicked, hovered, x, y, width, height, draw_x, draw_y, draw_width, draw_height = GuiGetPreviousWidgetInfo(gui)
  GuiOptionsAddForNextWidget(gui, GUI_OPTION.Layout_NoLayouting)
  GuiColorSetForNextWidget(gui, 0, 0, 0, 1)
  GuiZSetForNextWidget(gui, 1)
  GuiText(gui, x + 1, y + 1, amount_text)
  GuiLayoutEnd(gui)

  -- Draw invisible buttons on the button positions to catch mouse events because everything inside the
  -- scrollcontainer somehow ignores mouse events completely... hence this janky workaround
  GuiStartFrame(gui2)
  GuiImage(gui2, new_id(), board_x, board_y, "data/debug/whitebox.png", 0.0001, board_width / 20, board_height / 20)

  for i, v in ipairs(proto_click_catchers) do
    GuiImage(gui2, new_id(), v.x, v.y, "mods/banana_of_rectitude/files/skill_board/button_big.png", 0.00001, scale, scale)
    local clicked, right_clicked, hovered, x, y, width, height, draw_x, draw_y, draw_width, draw_height = GuiGetPreviousWidgetInfo(gui2)
    if hovered then
      GuiBeginAutoBox(gui2)
      GuiZSetForNextWidget(gui2, 1)
      GuiLayoutBeginVertical(gui2, v.x + 35, v.y, true)
      GuiText(gui2, 0, 0, get_bonus_tootlip(v.node.bonus))
      if v.node.activated then
        GuiText(gui2, 0, 10, "Unlocked!")
      else
        GuiLayoutBeginHorizontal(gui2, 0, 10, true)
        if perk_points < 1 then
          GuiColorSetForNextWidget(gui2, 0.8, 0, 0, 1)
        end
        GuiText(gui2, 0, 0, "Cost: 1x ")
        GuiImage(gui2, new_id(), 0, -2, "mods/banana_of_rectitude/files/banana_coin.png", 1, 1, 1)
        GuiLayoutEnd(gui2)
      end
      GuiLayoutEnd(gui2)
      GuiZSetForNextWidget(gui2, 2)
      GuiEndAutoBoxNinePiece(gui2)
    end
    if clicked and neighbor_is_active(v.node) and not v.node.activated and perk_points >= 1 then
      activate_node(v.node)
      perk_points = perk_points - 1
      ModSettingSet("banana_of_rectitude.perk_points", perk_points)
      GlobalsSetValue("banana_of_rectitude_perk_points", tostring(perk_points))
    end
  end

  GuiZSet(gui, 10)
  -- Controls, Zoom etc
  GuiBeginAutoBox(gui)
  local controls_x = board_x + board_width + 7
  local controls_y = board_y - 20
  GuiText(gui, controls_x, controls_y, ("Scale: %.2f"):format(scale))
  if GuiImageButton(gui, new_id(), controls_x + 50, controls_y, "", "mods/banana_of_rectitude/files/skill_board/button_zoom_in.png") then
    local old_scale = scale
    scale = math.min(2, scale * 1.25)
    scroll_offset_x, scroll_offset_y = scale_pos(scroll_offset_x, scroll_offset_y, old_scale, scale, board_x + board_width / 2, board_y + board_height / 2)
    constrain_offset()
  end
  GuiTooltip(gui, "Zoom in", "")
  if GuiImageButton(gui, new_id(), controls_x + 65, controls_y, "", "mods/banana_of_rectitude/files/skill_board/button_zoom_out.png") then
    local old_scale = scale
    scale = math.max(0.25, scale / 1.25)
    scroll_offset_x, scroll_offset_y = scale_pos(scroll_offset_x, scroll_offset_y, old_scale, scale, board_x + board_width / 2, board_y + board_height / 2)
    constrain_offset()
  end
  GuiTooltip(gui, "Zoom out", "")
  GuiZSet(gui, 11)
  GuiEndAutoBoxNinePiece(gui, 2, 0, 0)
  local clicked, right_clicked, hovered, x, y, width, height, draw_x, draw_y, draw_width, draw_height = GuiGetPreviousWidgetInfo(gui)
  -- Block mouse clicks while this panel is hovered
  if hovered then
    GuiImage(gui, new_id(), 0, 0, "data/debug/whitebox.png", 0.0001, 100, 100)
  end

  GuiZSet(gui, 10)
  -- Bonuses info box
  local padding = 2
  GuiBeginAutoBox(gui)
  GuiLayoutBeginVertical(gui, board_x + board_width + 5 + padding, board_y + padding, true)
  GuiColorSetForNextWidget(gui, 0.8, 0.8, 0, 1)
  GuiText(gui, 0, 0, "Active bonuses")
  for i, bonus_type in ipairs(display_order) do
    local bonus = bonuses[bonus_type]
    if bonus.count > 0 then
      if bonus_type == "CrypticStats" then
        GuiText(gui, 0, 0, b.cryptic_tooltip(nil, bonuses[bonus_type].display_text, sign_str(bonuses[bonus_type].count)))
      elseif bonus_type == "NonsenseStats" then
      else
        GuiText(gui, 0, 0, bonuses[bonus_type].display_text:format(sign_str(bonuses[bonus_type].value)))
      end
    end
  end
  GuiLayoutEnd(gui)
  GuiZSet(gui, 11)
  GuiEndAutoBoxNinePiece(gui, padding, 120, board_height - padding * 2)
  local clicked, right_clicked, hovered, x, y, width, height, draw_x, draw_y, draw_width, draw_height = GuiGetPreviousWidgetInfo(gui)
  -- Block mouse clicks while this panel is hovered
  if hovered then
    GuiImage(gui, new_id(), 0, 0, "data/debug/whitebox.png", 0.0001, 100, 100)
  end
end
