local utils = dofile_once("mods/banana_of_rectitude/files/utils.lua")

local function nonsense_tooltip(bonus, format_str, value)
  local word = bonus.nonsense_word
  if not word then
    utils.seed_with_frame_and_cam_pos()
    word = utils.get_random_element_sequence(utils.words)
    bonus.nonsense_word = word
  end
  return format_str:format(value, word)
end

local function cryptic_tooltip(bonus, format_str, value)
  if not random_string or utils.wait_frames("random_string", 5) then
    random_string = ""
    utils.seed_with_frame_and_cam_pos()
    for i=1, 10 do
      random_string = random_string .. string.char(65 + Random(60))
    end
  end
  return format_str:format(value, random_string)
end

local bonuses = {
  HealthFlat = { display_text = "%s Max HP", count = 0, value = 0 },
  HealthPercent = { display_text = "%s%% Max HP", count = 0, value = 0 },
  DamagePercent = { display_text = "%s%% Damage", count = 0, value = 0 },
  GoldPercent = { display_text = "%s%% Gold", count = 0, value = 0 },
  PotassiumPercent = { display_text = "%s%% Potassium", count = 0, value = 0 },
  DamageReductionPercent = { display_text = "%s%% Damage taken", count = 0, value = 0 },
  PlusPerk = { display_text = "Extra perk at holy mountains", count = 0, value = 0 },
  -- WandMoreSpells = { display_text = "%s Spells found on wands", count = 0, value = 0 },
  WandChanceExtraCapacity = { display_text = "%s%% Chance for +1 wand capacity", count = 0, value = 0 },
  WandPercentRechargeTime = { display_text = "%s%% Wand Recharge Time", count = 0, value = 0 },
  WandPercentCastDelay = { display_text = "%s%% Wand Cast Delay", count = 0, value = 0 },
  WandManaMax = { display_text = "%s%% Wand Mana Max", count = 0, value = 0 },
  WandManaRecharge = { display_text = "%s%% Wand Mana Charge Speed", count = 0, value = 0 },
  WandSpread = { display_text = "%s%% Wand Spread", count = 0, value = 0 },
  WandChanceNoShuffle = { display_text = "%s%% Chance to prevent wands from shuffling", count = 0, value = 0 },
  PotionCapacityPercent = { display_text = "%s%% Potion capacity", count = 0, value = 0 },
  MovementSpeedPercent = { display_text = "%s%% Movement speed", count = 0, value = 0 },
  NonsenseStats = { display_text = "%s %s", display_func = nonsense_tooltip, count = 0, value = 0 },
  CrypticStats = { display_text = "%s %s", display_func = cryptic_tooltip, count = 0, value = 0 },
}

local display_order = {
  "HealthFlat",
  "HealthPercent",
  "DamagePercent",
  "GoldPercent",
  "PotassiumPercent",
  "DamageReductionPercent",
  "PotionCapacityPercent",
  "MovementSpeedPercent",
  "WandChanceNoShuffle",
  "WandPercentCastDelay",
  "WandPercentRechargeTime",
  "WandManaMax",
  "WandManaRecharge",
  "WandChanceExtraCapacity",
  "WandSpread",
  "PlusPerk",
  "NonsenseStats",
  "CrypticStats",
}

local function get_activated_node_indexes()
  local data = ModSettingGet("banana_of_rectitude.activated_skill_nodes")
  if type(data) ~= "string" then
    return {}
  end
  local split = utils.split_string(data, ";")
  local out = {}
  for i, v in ipairs(split) do
    table.insert(out, v == "1")
  end
  return out
end

local function get_bonus(name)
  local value = tonumber(ModSettingGet("banana_of_rectitude.bonus_" .. name .. "_value")) or 0
  local count = tonumber(ModSettingGet("banana_of_rectitude.bonus_" .. name .. "_count")) or 0
  return value, count
end

local function set_bonus(name, value, count)
  ModSettingSet("banana_of_rectitude.bonus_" .. name .. "_count", count)
  ModSettingSet("banana_of_rectitude.bonus_" .. name .. "_value", value)
end

return {
  bonuses = bonuses,
  display_order = display_order,
  cryptic_tooltip = cryptic_tooltip,
  get_activated_node_indexes = get_activated_node_indexes,
  get_bonus = get_bonus,
  set_bonus = set_bonus,
}
