local function destroy_gui()
  if gui then
    GuiDestroy(gui)
    gui = nil
  end
end

if not GetValueBool("initialised", false) then
  -- execute_on_added
  SetValueBool("initialised", true)
else
  -- execute_on_removed
  destroy_gui()
end

local x = 0
local alpha = 0
local last_sound_played = 0
local start_frame = GameGetFrameNum()
local is_done = false

local function create_hole_of_size(x, y, r)
  hole_maker = EntityCreateNew("hole")
  EntitySetTransform(hole_maker, x, y)
  EntityAddComponent2(hole_maker, "CellEaterComponent", {
    radius=r
  })
  EntityAddComponent2(hole_maker, "LifetimeComponent", {
    lifetime=1
  })
end

local function get_spawn_pos(min_range, max_range, x, y)
  -- Try for a max of 100 times
  for i = 1, 100 do
    local angle = Random() * math.pi * 2
    local dx = x + (math.cos(angle) * Random(min_range, max_range))
    local dy = y + (math.sin(angle) * Random(min_range, max_range))
    local found_normal, normal_x, normal_y, approximate_distance_from_surface = GetSurfaceNormal(dx, dy, 16, 16)
    if not found_normal or approximate_distance_from_surface > 16 then
      -- Check if in ground
      local rhit, rx, ry = RaytracePlatforms(dx - 2, dy - 2, dx + 2, dy + 2)
      if not rhit then
        return dx, dy
      end
    end
  end
  create_hole_of_size(x, y, 32)
  return x, y
end

function wake_up_waiting_threads()
  gui = gui or GuiCreate()
  GuiStartFrame(gui)
  GuiOptionsAdd(gui, 2) -- NonInteractive

  local id = 1
  local new_id = function()
    id = id + 1
    return id
  end

  if start_frame < GameGetFrameNum() - 450 then
    is_done = true
  end
  if last_sound_played < GameGetFrameNum() - 40 then
    last_sound_played = GameGetFrameNum()
    GamePlaySound("mods/banana_of_rectitude/audio/banana_of_rectitude.bank", "warning_siren", 0, 0)
  end

  local screen_width, screen_height = GuiGetScreenDimensions(gui)
  local screen_center_y = screen_height / 2
  local band_width, band_height = GuiGetImageDimensions(gui, "mods/banana_of_rectitude/files/warning/band.png")
  GuiZSet(gui, 0)
  local top_band_y = screen_center_y - band_height / 2 - 50
  local bottom_band_y = screen_center_y - band_height / 2 + 50
  for i=1, math.ceil(screen_width / band_width) do
    GuiImage(gui, new_id(), -i * band_width + x, top_band_y, "mods/banana_of_rectitude/files/warning/band.png", 1, 1, 1)
    GuiImage(gui, new_id(), screen_width + (i-1) * band_width - x, bottom_band_y, "mods/banana_of_rectitude/files/warning/band.png", 1, 1, 1)
  end
  if not is_done then
    x = math.min(screen_width, x + 6)
  else
    x = x + 6
  end
  if x > screen_width * 2 + band_width then
    local entity_id = GetUpdatedEntityID()
    local player = EntityGetWithTag("player_unit")[1]
    local x, y = 0, 0
    if player then
      x, y = EntityGetTransform(player)
    else
      x, y = GameGetCameraPos()
    end
    local entity_id = GetUpdatedEntityID()
    SetRandomSeed(x + GameGetFrameNum(), y + GameGetFrameNum())
    local spawn_pos_x, spawn_pos_y = get_spawn_pos(0, 80, x, y - 20)
    EntityLoad("mods/banana_of_rectitude/files/superwizard_spawner.xml", spawn_pos_x, spawn_pos_y)
    EntityKill(entity_id)
  end
  local warning_text_img_width, warning_text_img_height = GuiGetImageDimensions(gui, "mods/banana_of_rectitude/files/warning/warning_text.png")
  alpha = 0.5 - math.cos((GameGetFrameNum() - start_frame) * 0.1) * 0.5
  GuiImage(gui, new_id(), screen_width / 2 - warning_text_img_width / 2, screen_center_y - warning_text_img_height / 2, "mods/banana_of_rectitude/files/warning/warning_text.png", alpha, 1, 1)

  if x == screen_width then
    local img_width, img_height = GuiGetImageDimensions(gui, "mods/banana_of_rectitude/files/warning/sign.png")
    GuiImage(gui, new_id(), 30, screen_center_y - img_height / 2, "mods/banana_of_rectitude/files/warning/sign.png", 1, 1, 1)
    GuiImage(gui, new_id(), screen_width - img_width - 30, screen_center_y - img_height / 2, "mods/banana_of_rectitude/files/warning/sign.png", 1, 1, 1)
    local text = "Level 69 Superwizard approaching"
    local text_width = GuiGetTextDimensions(gui, text) + 20
    for i=-1, math.ceil(screen_width / text_width) do
      -- Text "Superwizard approaching"
      GuiZSet(gui, -2)
      GuiText(gui, i * text_width + GameGetFrameNum() % text_width, top_band_y + 2, text)
      GuiText(gui, screen_width - (i * text_width + GameGetFrameNum() % text_width), bottom_band_y + 2, text)
      GuiZSet(gui, -1)
      -- Text shadow
      GuiColorSetForNextWidget(gui, 0, 0, 0, 1)
      GuiText(gui, i * text_width + GameGetFrameNum() % text_width + 1, top_band_y + 2 + 1, text)
      GuiColorSetForNextWidget(gui, 0, 0, 0, 1)
      GuiText(gui, screen_width - (i * text_width + GameGetFrameNum() % text_width) + 1, bottom_band_y + 2 + 1, text)
    end
  end
end
