local utils = dofile_once("mods/banana_of_rectitude/files/utils.lua")
local last_drag_x, last_drag_y, dragging_func, last_result
local function render_draggable(gui)
  if not dragging_func then return end
  -- Only render this once per frame
  if last_frame_rendered == GameGetFrameNum() then
    return last_result[1], last_result[2], last_result[3]
  end
  local mx, my = utils.get_mouse_gui_pos(gui)
  GuiOptionsAddForNextWidget(gui, GUI_OPTION.NoPositionTween)
  GuiOptionsAddForNextWidget(gui, GUI_OPTION.IsDraggable)
  GuiOptionsAddForNextWidget(gui, GUI_OPTION.IsExtraDraggable)
  GuiOptionsAddForNextWidget(gui, GUI_OPTION.NoSound)
  GuiOptionsAddForNextWidget(gui, GUI_OPTION.ClickCancelsDoubleClick)
  GuiOptionsAddForNextWidget(gui, GUI_OPTION.DrawNoHoverAnimation)
  GuiIdPushString(gui, "draggable")
  local draw_x, draw_y = mx - 25, my - 25
  -- GuiImageButton(gui, 2, mx - 28, my - 28, "", "data/debug/circle_56.png")
  GuiImageButton(gui, 2, draw_x, draw_y, "", "mods/banana_of_rectitude/lib/EZMouse/invis.png")
  GuiIdPop(gui)
  local _, _, _, _, _, _, _, dx, dy = GuiGetPreviousWidgetInfo(gui)
  local moved_by_x, moved_by_y = 0, 0
  if (not utils.are_floats_equal(dx, draw_x) or not utils.are_floats_equal(dy, draw_y)) and dx ~= 0 and dy ~= 0 then
    moved_by_x = (dx - (last_drag_x or dx))
    moved_by_y = (dy - (last_drag_y or dy))
    if dragging_func then
      dragging_func(moved_by_x, moved_by_y, not not last_drag_x)
    end
    last_drag_x = dx
    last_drag_y = dy
    last_result = { true, moved_by_x, moved_by_y }
    return true, moved_by_x, moved_by_y
  end
  -- Drag end
  last_drag_x = nil
  last_drag_y = nil
  dragging_func = nil
  last_result = { false }
  return last_result
end

local windows = {}
local window_mt = {}
function window_mt.__index(self, key)
  return window_mt[key]
end

local window_to_focus
function window_mt:focus()
  window_to_focus = self
end
local window_to_close
function window_mt:close()
  window_to_close = self
end

function window_mt:update(gui, new_id, mouse_event)
  if not self.button_width then
    self.button_width, self.button_height = GuiGetImageDimensions(gui, "mods/banana_of_rectitude/files/windows_button_ok.png")
    self.button_x = self.width / 2 - self.button_width / 2
    self.button_y = self.height - self.button_height - 10
  end
  if not self.close_button_width then
    self.close_button_width, self.close_button_height = GuiGetImageDimensions(gui, "mods/banana_of_rectitude/files/windows_button_close.png")
    self.close_button_x = self.width - self.close_button_width - 5
    self.close_button_y = 5
  end
  if not mouse_event then return end
  local titlebar_height = 21
  if mouse_event.y <= self.y + titlebar_height and mouse_event.x < self.x + self.width - 23 then
    if not dragging_func then
      dragging_func = function(dx, dy, drag_started)
        self.x = self.x + dx
        self.y = self.y + dy
        if drag_started then
          self:focus()
        end
      end
    end
  end

  if mouse_event.left_just_down then
    self:focus()
  end

  if mouse_event.left_just_up then
    if utils.is_point_in_rect(mouse_event.x, mouse_event.y, self.x + self.button_x, self.y + self.button_y, self.button_width, self.button_height) then
      if type(self.on_ok) == "function" then
        self.on_ok()
      end
    end
    if utils.is_point_in_rect(mouse_event.x, mouse_event.y, self.x + self.close_button_x, self.y + self.close_button_y, self.close_button_width, self.close_button_height) then
      self:close()
    end
  end
end

function window_mt:render(gui, new_id, mouse_event)
  local mx, my = utils.get_mouse_gui_pos(gui)
  local np_size = 21
  -- Window/Background
  GuiZSetForNextWidget(gui, self.z)
  GuiImageNinePiece(gui, new_id(), self.x + np_size, self.y + np_size, self.width - np_size * 2, self.height - np_size * 2, 1, "mods/banana_of_rectitude/files/windows_window.png", "mods/banana_of_rectitude/files/windows_window.png")
  -- Text
  GuiZSetForNextWidget(gui, self.z - 1)
  GuiColorSetForNextWidget(gui, 0, 0, 0, 1)
  GuiText(gui, self.x + 50, self.y + self.height / 2 - 16, self.text)
  -- OK Button
  local button_down = mouse_event and utils.is_point_in_rect(mouse_event.x, mouse_event.y, self.x + self.button_x, self.y + self.button_y, self.button_width, self.button_height) and mouse_event.left_down
  local button_img = "mods/banana_of_rectitude/files/windows_button_ok.png"
  if self.button_focused then
    button_img = "mods/banana_of_rectitude/files/windows_button_ok_focused.png"
  end
  if button_down then
    self.button_focused = true
    button_img = "mods/banana_of_rectitude/files/windows_button_ok_down.png"
  end
  GuiZSetForNextWidget(gui, self.z - 1)
  GuiImage(gui, new_id(), self.x + self.button_x, self.y + self.button_y, button_img, 1, 1, 1)
  -- Close button
  local button_down = mouse_event and utils.is_point_in_rect(mouse_event.x, mouse_event.y, self.x + self.close_button_x, self.y + self.close_button_y, self.close_button_width, self.close_button_height) and mouse_event.left_down
  local button_img = "mods/banana_of_rectitude/files/windows_button_close.png"
  if button_down then
    button_img = "mods/banana_of_rectitude/files/windows_button_close_down.png"
  end
  GuiZSetForNextWidget(gui, self.z - 1)
  GuiImage(gui, new_id(), self.x + self.close_button_x, self.y + self.close_button_y, button_img, 1, 1, 1)
  -- Title text
  GuiZSetForNextWidget(gui, self.z - 1)
  GuiText(gui, self.x + 10, self.y + 7, self.title)
  -- Icon
  local icon_width, icon_height = GuiGetImageDimensions(gui, "mods/banana_of_rectitude/files/windows_icon_warning.png")
  GuiZSetForNextWidget(gui, self.z - 1)
  GuiImage(gui, new_id(), self.x + 10, self.y + self.height / 2 - icon_height / 2, "mods/banana_of_rectitude/files/windows_icon_warning.png", 1, 1, 1)
end

local function create_window(x, y, title)
  local window = setmetatable({
    x = x,
    y = y,
    z = #windows * -5,
    width = 200,
    height = 100,
    title = title or "",
    text = "",
  }, window_mt)
  table.insert(windows, window)
  return window
end

local function update_window_system(gui, new_id)
  front_most_mouse_event = { z = math.huge }
  -- Collect mouse events from foremost to backmost
  local mx, my = utils.get_mouse_gui_pos(gui)
  local mouse_event
  for i=#windows, 1, -1 do
    local window = windows[i]
    if not mouse_event and utils.is_point_in_rect(mx, my, window.x, window.y, window.width, window.height) then
      -- Block mouse clicks
      GuiOptionsAddForNextWidget(gui, GUI_OPTION.NoPositionTween)
      GuiImage(gui, new_id(), mx - 28, my - 28, "data/debug/circle_56.png", 0.001, 1, 1)
      mouse_event = {
        window = window,
        x = mx, y = my,
        left_down = InputIsMouseButtonDown(1),
        left_just_down = InputIsMouseButtonJustDown(1),
        left_just_up = InputIsMouseButtonJustUp(1),
      }
    end
  end

  -- Update from foremost to backmost
  for i=#windows, 1, -1 do
    local window = windows[i]
    window:update(gui, new_id, mouse_event and mouse_event.window == window and mouse_event or false)
  end

  for i, window in ipairs(windows) do
    window:render(gui, new_id, mouse_event and mouse_event.window == window and mouse_event or false)
  end

  if window_to_focus then
    local index_of_this = -1
    for i, window in ipairs(windows) do
      if window == window_to_focus then
        index_of_this = i
      else
        window.button_focused = false
      end
    end
    local window = table.remove(windows, index_of_this)
    table.insert(windows, window)
    for i, window in ipairs(windows) do
      window.z = (i-1) * -5
    end
    window_to_focus = nil
  end

  if window_to_close then
    for i, window in ipairs(windows) do
      if window == window_to_close then
        if type(window.on_close) == "function" then
          window.on_close()
        end
        table.remove(windows, i)
      end
    end
    table.sort(windows, function (a, b)
      return a.z > b.z
    end)
    for i, window in ipairs(windows) do
      window.z = (i-1) * -5
    end
    window_to_close = nil
  end
  render_draggable(gui)
end

return {
  create_window = create_window,
  update_window_system = update_window_system,
  windows = windows,
}