--[[
                    ██████
                    ██▓▓▓▓██
                    ██▒▒▓▓██
                    ██▒▒▓▓  ████
                      ██        ██
                      ██        ░░██
                      ██        ░░██
                    ██        ░░░░██
                    ██        ░░░░██
                    ██        ░░░░██
                  ██          ░░░░██
              ████            ░░░░██
  ████████████              ░░░░██
██▒▒                      ░░░░██
██▒▒                    ░░░░██
██▓▓              ░░░░░░░░██
  ██████░░░░░░░░░░░░░░████
        ██████████████
]]
local skill_bonuses = dofile_once("mods/banana_of_rectitude/files/skill_board/skill_bonuses.lua")
local rectitude_meter = dofile_once("mods/banana_of_rectitude/files/rectitude_meter.lua")
local bonus_skill = dofile_once("mods/banana_of_rectitude/files/bonus_skill_apply_functions.lua")
local window_system = dofile_once("mods/banana_of_rectitude/files/windows.lua")
local utils = dofile_once("mods/banana_of_rectitude/files/utils.lua")
dofile_once("mods/banana_of_rectitude/lib/DialogSystem/init.lua")("mods/banana_of_rectitude/lib/DialogSystem", {
  typing_sound = "one",
  sounds = {
    banana = { bank = "mods/banana_of_rectitude/audio/banana_of_rectitude.bank", event = "typing_sounds/banana" },
    monke = { bank = "mods/banana_of_rectitude/audio/banana_of_rectitude.bank", event = "typing_sounds/monke" },
    amogus = { bank = "mods/banana_of_rectitude/audio/banana_of_rectitude.bank", event = "amogus" },
  }
})

local world_blessed_with_potassium = HasFlagPersistent("banana_of_rectitude_world_blessed_with_potassium")

ModRegisterAudioEventMappings("mods/banana_of_rectitude/audio/GUIDs.txt")
ModMaterialsFileAdd("mods/banana_of_rectitude/files/materials.xml")
ModLuaFileAppend("data/scripts/gun/gun_actions.lua", "mods/banana_of_rectitude/files/gun_actions_append.lua")
if not world_blessed_with_potassium then
  ModLuaFileAppend("data/scripts/biomes/snowcastle_cavern.lua", "mods/banana_of_rectitude/files/snowcastle_cavern_append.lua")
  ModLuaFileAppend("data/scripts/biome_scripts.lua", "mods/banana_of_rectitude/files/biome_scripts_append.lua")
end

-- Add banana drop script to basic humanoids
;(function()
  local content = ModTextFileGetContent("data/entities/base_humanoid.xml")
  content = content:gsub("<GenomeDataComponent", [[
    <LuaComponent
      execute_every_n_frame="-1"
      script_death="mods/banana_of_rectitude/files/base_humanoid_death.lua"
      remove_after_executed="1"
    ></LuaComponent>
    <GenomeDataComponent
  ]])
  ModTextFileSetContent("data/entities/base_humanoid.xml", content)
end)()

if world_blessed_with_potassium then
  -- Turn some worms into bananas
  (function()
    local content = ModTextFileGetContent("data/entities/animals/worm.xml")
    content = content:gsub("</Entity>", [[
      <LuaComponent
        script_source_file="mods/banana_of_rectitude/files/worm_init.lua"
        execute_every_n_frame="-1"
        execute_on_added="1"
        remove_after_executed="1"
      ></LuaComponent>
      </Entity>
    ]])
    ModTextFileSetContent("data/entities/animals/worm.xml", content)
  end)()

  -- Turn some big worms into bananas
  ;(function()
    local content = ModTextFileGetContent("data/entities/animals/worm_big.xml")
    content = content:gsub("</Entity>", [[
      <LuaComponent
        script_source_file="mods/banana_of_rectitude/files/worm_big_init.lua"
        execute_every_n_frame="-1"
        execute_on_added="1"
        remove_after_executed="1"
      ></LuaComponent>
      </Entity>
    ]])
    ModTextFileSetContent("data/entities/animals/worm_big.xml", content)
  end)()
end

local debug_menu = {
  Items = {
    "mods/banana_of_rectitude/files/bananomaly_detector.xml",
    "mods/banana_of_rectitude/files/bananaputer.xml",
    "mods/banana_of_rectitude/files/disk_banana.xml",
    "mods/banana_of_rectitude/files/disk_banana2.xml",
    "mods/banana_of_rectitude/files/disk_virus.xml",
    "mods/banana_of_rectitude/files/disk_rectitude.xml",
    "mods/banana_of_rectitude/files/disk_shootemup.xml",
    "mods/banana_of_rectitude/files/banana_hearthstone.xml",
    "mods/banana_of_rectitude/files/banana_grenade.xml",
    "mods/banana_of_rectitude/files/banana_phone.xml",
    "mods/banana_of_rectitude/files/bananaputer.xml",
    "mods/banana_of_rectitude/files/skill_board/skill_board_item.xml",
  },
  Entities = {
    "mods/banana_of_rectitude/files/bananomaly_detector.xml",
    "mods/banana_of_rectitude/files/dancing_banana.xml",
    "mods/banana_of_rectitude/files/banana_portal.xml",
    "mods/banana_of_rectitude/files/superwizard_spawner.xml",
    "mods/banana_of_rectitude/files/superwizard_approaching.xml",
    "mods/banana_of_rectitude/files/dont_feed_monkey_sign.xml",
    "mods/banana_of_rectitude/files/lemonke_caged.xml",
    "mods/banana_of_rectitude/files/banana_phone.xml",
    "data/entities/animals/miner.xml",
    "data/entities/items/pickup/heart.xml",
    "data/entities/items/pickup/heart_better.xml",
  },
  -- Custom = {
  --   Window = function(gui)
  --     local screen_width, screen_height = GuiGetScreenDimensions(gui)
  --     local x = Random(0, screen_width - 200)
  --     local y = Random(0, screen_height - 60)
  --     local window = window_system.create_window(x, y, "Warning")
  --     window.text = "You really shouldn't have done that."
  --     local a = GameGetFrameNum()
  --     window.on_ok = function()
  --       window:close()
  --     end
  --   end
  -- }
}
local function render_debug_menu(gui, new_id)
  local x, y = 20, 50
  GuiLayoutBeginVertical(gui, x, y, true)
  for k, v in pairs(debug_menu) do
    if GuiButton(gui, new_id(), 0, 0, k) then
      v.open = not v.open
    end
    if v.open then
      for i, v2 in ipairs(v) do
        if GuiButton(gui, new_id(), 0, 0, "  " .. v2) then
          if k == "Entities" then
            local x, y = GameGetCameraPos()
            EntityLoad(v2, x + 30, y)
          else
            local player = EntityGetWithTag("player_unit")[1]
            if player then
              local x, y = EntityGetTransform(player)
              GamePickUpInventoryItem(player, EntityLoad(v2, x + 30, y), false)
            end
          end
        end
      end
    end
  end
  if GuiButton(gui, new_id(), 0, 0, "Giv wand") then
    local EZWand = dofile_once("mods/banana_of_rectitude/lib/EZWand.lua")
    local wand = EZWand{
      rechargeTime = -60,
      castDelay = -60,
      manaChargeSpeed = -60,
      manaMax = 10,
      spread = -10,
    }
    wand:PutInPlayersInventory()
  end
  if GuiButton(gui, new_id(), 0, 0, "Give bon") then
    GlobalsSetValue("banana_of_rectitude_WandPercentRechargeTime_value", "-20")
    GlobalsSetValue("banana_of_rectitude_WandPercentCastDelay_value", "-20")
    GlobalsSetValue("banana_of_rectitude_WandManaMax_value", "20")
    GlobalsSetValue("banana_of_rectitude_WandManaRecharge_value", "20")
    GlobalsSetValue("banana_of_rectitude_WandSpread_value", "-20")
    GlobalsSetValue("banana_of_rectitude_WandChanceNoShuffle_value", "20")
    GlobalsSetValue("banana_of_rectitude_WandChanceExtraCapacity_value", "100")
    GlobalsSetValue("banana_of_rectitude_PotassiumPercent_value", "50")
    GlobalsSetValue("banana_of_rectitude_PotionCapacityPercent_value", "50")
    GlobalsSetValue("banana_of_rectitude_MovementSpeedPercent_value", "20")
  end
  GuiLayoutEnd(gui)
end

-- Oh god what a clusterfuck
dofile_once("mods/banana_of_rectitude/files/skill_board/json.lua")
local b = dofile_once("mods/banana_of_rectitude/files/skill_board/skill_bonuses.lua")
local bonuses = {}
for i, v in ipairs(b.display_order) do
  bonuses[v] = { value = 0, count = 0 }
end
local data = JSON:decode(dofile_once("mods/banana_of_rectitude/files/skill_board/nodes.lua"))
local nodes = data.nodes
for i, active in ipairs(skill_bonuses.get_activated_node_indexes()) do
  if active then
    local bonus_index = nodes[i].bonus + 1
    local bonus_type = data.bonuses[bonus_index].type
    local bonus_value = data.bonuses[bonus_index].value or 0
    bonuses[bonus_type].value = bonuses[bonus_type].value + bonus_value
    bonuses[bonus_type].count = bonuses[bonus_type].count + 1
  end
end
for k, v in pairs(bonuses) do
  b.set_bonus(k, v.value, v.count)
end

function OnPlayerSpawned(player)
  if GameHasFlagRun("banana_of_rectitude_init") then return end
  GameAddFlagRun("banana_of_rectitude_init")
  local x, y = EntityGetTransform(player)
  -- create_banana_wand():PutInPlayersInventory()
  local perk_points = ModSettingGet("banana_of_rectitude.perk_points") or 0
  -- Save perk points in globals because mod setting get uses file access I guess and might be slow when used every frame
  GlobalsSetValue("banana_of_rectitude_perk_points", tostring(perk_points))
  GlobalsSetValue("banana_of_rectitude_coins_redeemed", "0")
  GlobalsSetValue("banana_of_rectitude_banana_x", "305")
  GlobalsSetValue("banana_of_rectitude_banana_y", "-90")

  local seed = utils.create_seed()
  SetRandomSeed(seed, seed)
  if Randomf() < 0.2 then
    GameAddFlagRun("banana_of_rectitude_superwizard_enabled")
  end
  GlobalsSetValue("banana_of_rectitude_bananomaly_target_x", tostring(Random(-10000, 10000)))
  GlobalsSetValue("banana_of_rectitude_bananomaly_target_y", tostring(Random(0, 10000)))

  EntityLoad("mods/banana_of_rectitude/files/banana.xml", 305, -90)
  if world_blessed_with_potassium then
    rectitude_meter.set_rectitude(player, 0)
  end
  local value, count = skill_bonuses.get_bonus("NonsenseStats")
  if count >= 10 then
    local world_state_entity = GameGetWorldStateEntity()
    local world_state_comp = EntityGetFirstComponentIncludingDisabled(world_state_entity, "WorldStateComponent")
    if world_state_comp then
      ComponentSetValue2(world_state_comp, "material_everything_to_gold", "banana_of_rectitude_potassium")
      ComponentSetValue2(world_state_comp, "material_everything_to_gold_static", "banana_of_rectitude_potassium")
    end
  end

  EntityAddComponent2(player, "LuaComponent", {
    script_source_file="mods/banana_of_rectitude/files/consume_potassium.lua",
    execute_every_n_frame=5,
  })
  bonus_skill.apply_movement_bonus(player)
  bonus_skill.apply_health_bonus(player)
  bonus_skill.apply_damage_bonus(player)
  bonus_skill.apply_damage_reduction_bonus(player)
  bonus_skill.apply_wand_and_potion_bonus(player)
end

bonus_skill.apply_gold_bonus()
bonus_skill.apply_perk_bonus()
bonus_skill.apply_health_percent_bonus_to_hearts()

local bananometer_x, bananometer_y
local EZMouse = dofile_once("mods/banana_of_rectitude/lib/EZMouse/EZMouse.lua")("mods/banana_of_rectitude/lib/EZMouse/")
function draw_bananometer(gui, new_id, current_fill)
  local max_fill = rectitude_meter.max_rectitude
  local width = 8
  local height = 67
  local inner_width = 2
  local inner_height = 52
  local screen_width, screen_height = GuiGetScreenDimensions(gui)
  if not bananometer_x then
    bananometer_x = ModSettingGet("banana_of_rectitude.bananometer_x") or (screen_width - width - 4)
    bananometer_y = ModSettingGet("banana_of_rectitude.bananometer_y") or 40
  end
  if not bananometer_widget then
    bananometer_widget = EZMouse.Widget({
      x = bananometer_x - 2,
      y = bananometer_y,
      z = 1,
      width = width + 4, height = height,
      draggable = true, enabled = true,
      constraints = { left = 0, top = 0, right = screen_width, bottom = screen_height }
    })
    bananometer_widget:AddEventListener("drag", function(self, event)
      bananometer_x = self.x + 2
      bananometer_y = self.y
    end)
    bananometer_widget:AddEventListener("drag_end", function(self, event)
      ModSettingSet("banana_of_rectitude.bananometer_x", self.x + 2)
      ModSettingSet("banana_of_rectitude.bananometer_y", self.y)
    end)
  end

  GuiZSet(gui, 2)
  GuiImage(gui, new_id(), bananometer_x, bananometer_y, "mods/banana_of_rectitude/files/bananometer.png", 1, 1, 1)
  GuiZSet(gui, 1)
  -- Work around the fact that if scale_y is 0 it gets treated as 1 by the engine (because of default value shenanigans)
  if current_fill == 0 then
    current_fill = 0.00001
  end
  local percent_fill = current_fill / max_fill
  local sub_image_scale_y = percent_fill * inner_height / inner_height
  GuiImage(gui, new_id(), bananometer_x + 3, bananometer_y + 10 + (1 - percent_fill) * inner_height, "mods/banana_of_rectitude/files/bananometer_fill.png", 1, 1, sub_image_scale_y)
  GuiTextCentered(gui, bananometer_x + 4, bananometer_y + 68, tostring(math.floor(current_fill)))
end

local text_in_world = {}
local function add_text_to_world(world_x, world_y, text)
  table.insert(text_in_world, {
    time_spawned = GameGetFrameNum(),
    text = text,
    world_x = world_x,
    world_y = world_y,
    phase = 0,
  })
end

local function render_text_in_world(gui)
  local to_remove = {}

  for i, v in ipairs(text_in_world) do
    if not v.text_width then
      v.text_width, v.text_height = GuiGetTextDimensions(gui, v.text)
    end
    local screen_x, screen_y = utils.world_to_screen_coords(gui, v.world_x, v.world_y)
    screen_x = screen_x - v.text_width / 2
    screen_y = screen_y - v.text_height / 2
    v.phase = v.phase + 0.08
    local frames_alive = GameGetFrameNum() - v.time_spawned
    local alpha = utils.map(frames_alive, 0, 240, 0, math.pi * 2)
    alpha = 0.5 + math.cos(alpha + math.pi) *  0.5
    if alpha == 0 then
      alpha = 0.0001
    end
    local cur_x = screen_x
    for i=1, #v.text do
      local char = v.text:sub(i, i)
      i = i + 1
      local x, y = cur_x, screen_y
      y = y + math.sin(v.phase + i * 0.3) * 2
      GuiColorSetForNextWidget(gui, 1, 1, 1, alpha)
      GuiText(gui, x, y, char)
      cur_x = cur_x + GuiGetTextDimensions(gui, char)
    end
    if frames_alive >= 240 - 1 then
      table.insert(to_remove, i)
    end
  end
  for i, v in ipairs(to_remove) do
    table.remove(text_in_world, v)
  end
end

local function get_random_text()
  local cam_x, cam_y = GameGetCameraPos()
  SetRandomSeed(cam_x + GameGetFrameNum(), cam_y)
  return utils.get_random_element_sequence(utils.words)
end

local function spawn_window(gui)
  local screen_width, screen_height = GuiGetScreenDimensions(gui)
  local x = Random(0, screen_width - 200)
  local y = Random(0, screen_height - 60)
  local window = window_system.create_window(x, y, "Warning")
  window.text = "You shouldn't have done that."
  window.on_ok = function()
    window:close()
  end
end

local last_frame_virus_window_spawned = 0
local next_virus_window_spawn_delay = 0

local function load_le_monke()
  local monke_x, monke_y = 97, -250
  if DoesWorldExistAt(monke_x - 200, monke_y - 200, monke_x + 200, monke_y + 200) then
    local did_hit, hit_x, hit_y = RaytraceSurfaces(monke_x, monke_y, monke_x, monke_y + 250)
    EntityLoad("mods/banana_of_rectitude/files/lemonke_freed.xml", hit_x, hit_y)
    GameAddFlagRun("banana_of_rectitude_lemonke_spawned")
  end
end

function OnWorldPostUpdate()
  if not GameHasFlagRun("banana_of_rectitude_lemonke_spawned") and world_blessed_with_potassium then
    load_le_monke()
  end
  local frame_num = GameGetFrameNum()
  local cam_x, cam_y = GameGetCameraPos()
  SetRandomSeed(frame_num + cam_x, cam_y)
  local id = 1
  local function new_id()
    id = id + 1
    return id
  end

  gui = gui or GuiCreate()
  GuiStartFrame(gui)

  EZMouse.update(gui)

  GuiZSet(gui, 100)
  render_text_in_world(gui)
  -- render_debug_menu(gui, new_id)

  if false and GuiButton(gui, new_id(), 10, 200, "Give perk points") then
    ModSettingSet("banana_of_rectitude.perk_points", 3)
    GlobalsSetValue("banana_of_rectitude_perk_points", tostring(3))
  end

  local player = EntityGetWithTag("player_unit")[1]
  if player then
    -- local px, py = EntityGetTransform(player)
    -- if GuiButton(gui, new_id(), 10, 200, "Give banan") then
    --   rectitude_meter.set_rectitude(player, 100)
    --   -- EntityIngestMaterial(player, CellFactory_GetType("meat_banana"), 1100)
    -- end
    -- local controls_component = EntityGetFirstComponentIncludingDisabled(player, "ControlsComponent")
    -- if controls_component then
    --   if ComponentGetValue2(controls_component, "mButtonFrameRightClick") == GameGetFrameNum() then
    --     local mx, my = DEBUG_GetMouseWorld()
    --     -- EntityLoad("mods/banana_of_rectitude/files/banana_physics_prop.xml", mx, my)
    --     EntityLoad("data/entities/animals/worm_big.xml", mx, my)
    --     -- EntityLoad("mods/banana_of_rectitude/files/skill_board/skill_board_item.xml", mx, my)
    --     -- EntityLoad("mods/banana_of_rectitude/files/lemonke_caged.xml", mx, my)
    --   end
    -- end

    last_frame_rectitude = last_frame_rectitude or 0
    if rectitude_meter.has_rectitude_unlocked(player) then
      local rectitude = rectitude_meter.get_rectitude(player)
      if rectitude > last_frame_rectitude then
        if utils.wait_frames("text", 20) then
          local cam_x, cam_y, cam_width, cam_height = GameGetCameraBounds()
          SetRandomSeed(cam_x + GameGetFrameNum(), cam_y)
          add_text_to_world(Random(cam_x, cam_x + cam_width), Random(cam_y, cam_y + cam_height), get_random_text())
        end
        last_frame_rectitude = rectitude
      end
      draw_bananometer(gui, new_id, rectitude)
    end

    if utils.wait_frames("scramble_text", 10 - #window_system.windows + 2) then
      for i, window in ipairs(window_system.windows) do
        if i > 2 then
          window.orig_text = window.orig_text or window.text
          window.text = ""
          for i=1, #window.orig_text do
            local ch = window.orig_text:sub(i, i)
            if Random() < 0.2 then
              ch = string.upper(ch)
              -- ch = string.char(Random(65, 120))
            end
            window.text = window.text .. ch
          end
        end
      end
    end
  end

  window_system.update_window_system(gui, new_id)
  dofile_once("mods/banana_of_rectitude/files/ascii_banana/ascii_banana.lua")(gui)
  dofile_once("mods/banana_of_rectitude/files/3d_banana.lua")(gui, new_id)

  if GlobalsGetValue("banana_of_rectitude_virus", "0") == "1" then
    if last_frame_virus_window_spawned + next_virus_window_spawn_delay < frame_num then
      last_frame_virus_window_spawned = frame_num
      next_virus_window_spawn_delay = Random(60, 3600) -- 1 second to 1 minute
      spawn_window(gui)
    end
  end

  if GameHasFlagRun("banana_of_rectitude_superwizard_enabled") then
    -- Spawn superwizard randomly after at least 15 minutes playtime, 5% chance for it to spawn per minute
    if (frame_num > 60 * 60 * 15) and (frame_num % (60 * 60) == 0) then
      if (Randomf() < 0.05) and GlobalsGetValue("banana_of_rectitude_superwizard_spawned", "0") == "0" then
        GlobalsSetValue("banana_of_rectitude_superwizard_spawned", "1")
        EntityLoad("mods/banana_of_rectitude/files/superwizard_approaching.xml", cam_x, cam_y)
      end
    end
  end
end

local shootemup = dofile_once("mods/banana_of_rectitude/files/shootemup.lua")
function OnPausePreUpdate()
  if GameHasFlagRun("banana_of_rectitude_shootemup") then
    GuiStartFrame(gui)
    shootemup.OnPausePreUpdate(gui)
  end
end
function OnPausedChanged(...)
  if GameHasFlagRun("banana_of_rectitude_shootemup") then
    shootemup.OnPausedChanged(...)
  end
end

function OnWorldInitialized()
  -- Since we can't get the world seed before OnWorldInitialized, store THIS runs seed for the RNG seed for next run
  ModSettingSet("banana_of_rectitude.random_seed", tonumber(StatsGetValue("world_seed")) or 0)
  GlobalsSetValue("banana_of_rectitude_GoldPercent_value", skill_bonuses.get_bonus("GoldPercent"))
end

if ModSettingGet("banana_of_rectitude.virus_active") and ModImageMakeEditable ~= nil then
  ModSettingRemove("banana_of_rectitude.virus_active")
  dofile_once("mods/banana_of_rectitude/files/corruption.lua")
end
